﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Runtime.InteropServices;
using System.Windows;
using RTedge;   // RT-edge system API
using EdgeCode; // RT-edge template code

namespace EgExtIF
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        edgeCode EgProc = null;                 //RTedge フレームワーク+APIを使用したコードクラス
        EgUser EgUser = null;               //ユーザー実装クラス

        readonly int MAX_PATH_LENGTH = 256;          //パス最大長
        public MainWindow()
        {
            InitializeComponent();              //Componentの初期化(自動生成コード)
            //AllocConsole();                     //[DEBUG用] コンソール出力用
            //MessageBox.Show("debug");
            EgProc = new edgeCode();            //RTedge処理クラスの作成
            EgUser = new EgUser(EgProc.EGAPI, EgProc.mEgFwDesc, EgProc.STATE, EgProc.EGCONST);    //ユーザー処理クラスの生成
            EgProc.mInstMainClose = Stop;       //フレームワーク内終了メッセージ処理時の関数指定

            fnGetCommandLine();                 //コマンドライン引数取得

            EgProc.AddUserFunc(EgUser.egFwCBs); //ユーザー実装関数登録
            EgProc.InitService();               //RTedge処理クラスの初期化
        }
        /// <summary>
        /// コマンドライン取得
        /// </summary>
        public void fnGetCommandLine()
        {
            try
            {
                string[] args = Environment.GetCommandLineArgs();   //コマンドライン取得
                if (args.Length > 0) //指定があれば中に入る
                {
                    bool bDllFlag = false;
                    //文字列解析
                    for (int i = 0; i < args.Length; i++)
                    {

                        string arg = args[i];
                        if (arg.Equals("-dll"))    // -dll の引数がある場合はdll名を取得する
                        {
                            bDllFlag = true;
                            continue;
                        }
                        if (bDllFlag == true)       // -dllがあった場合は次の引数がパス
                        {
                            bDllFlag = false;
                            if (arg.Length < MAX_PATH_LENGTH + 1)//パスの長さ最大256文字以下の場合書対象とする
                                EgProc.mEgFwDesc.stDLLPath = arg;
                            else
                            {
                                EgProc.mEgFwDesc.stDLLPath = "";
                                //最大パス長より長い
                                EgProc.LOG(edge_FW.EG_LOG_ERR, "The specified dll path length is too long.");
                            }
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                EgProc.LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// サービス終了処理
        /// </summary>
        public void Stop()
        {
            try
            {
                Dispatcher.Invoke((Action)(() =>
                {
                    Close();                        //Window Close関数呼び出し
                }));
            }
            catch (Exception ex)
            {
                EgProc.LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            ;
        }
        /// <summary>
        /// Windowクローズ処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            try
            {
                EgProc.KillService();           //サービス削除関数呼び出し<必須>
                EgProc.EGAPI.EgFinalize();      //APIのFinalize処理追加 <必須>
                EgProc.Release();               //後処理
                //[TODO] : 必要に応じて終了処理追加
            }
            catch (Exception ex)
            {
                EgProc.LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// [DEBUG用] コンソール出力するための定義
        /// </summary>
        /// <returns></returns>
        [DllImport("kernel32.dll", SetLastError = true)]
        [return: MarshalAs(UnmanagedType.Bool)]
        static extern bool AllocConsole();
    }
}
