﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using RTedge;   // RT-edge system API
using ExtCommon;

namespace EdgeCode
{
    /// <summary>
    /// デバイス固有情報
    /// </summary>
    public class EgDevDesc
    {
        //
        // TDOO: 必要に応じて各種追加
        //
    }

    /// <summary>
    /// ユーザー固有処理クラス
    /// </summary>
    public class EgUser
    {
        public edge_API EGAPI = null;                               // edge APIのインスタンス
        public MYSTATE STATE = null;                       // サービス状態クラスのインスタンス
        public EDGECONST EGCONST = null;                            // 各種定義値
        public EgFwCBs egFwCBs = null;                              // ユーザー実装関数群
        public EgDevDesc egDevDesc = null;                          // ユーザー固有定義
        public EgFWdesc EGFWDESC = null;                            // Edgeフレームワーク共通IF

        ///*****************************************************************************
        /// <summary>
        /// コンストラクタ
        /// </summary>
        /// <param name="egapi">edge APIのインスタンス</param>
        /// <param name="egFWdesc">Edgeフレームワーク共通IF</param>
        /// <param name="mystate">サービス状態クラスのインスタンス</param>
        /// <param name="egconst">各種定義値</param>
        ///*****************************************************************************
        public EgUser(edge_API egapi, EgFWdesc egFWdesc, MYSTATE mystate, EDGECONST egconst)
        {
            EGAPI = egapi;
            STATE = mystate;
            EGCONST = egconst;
            EGFWDESC = egFWdesc;            // Edgeフレームワーク共通IF
            egDevDesc = new EgDevDesc();    // デバイス固有情報のインスタンス生成
            egFwCBs = SetCallback();        // 必須のコールバック関数をセット
        }

        ///*****************************************************************************
        /// <summary>
        /// コールバック関数の登録[必須]
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public EgFwCBs SetCallback()
        {
            // [必須]コールバック関数の登録
            EgFwCBs cbs = new EgFwCBs();
            cbs.cbInit = CB_INIT;
            cbs.cbOpen = CB_OPEN;
            cbs.cbClose = CB_CLOSE;
            cbs.cbRead = CB_READ;
            cbs.cbReadEx = CB_READEX;
            cbs.cbReadExAll = CB_READEXALL;
            cbs.cbWrite = CB_WRITE;
            cbs.cbWriteEx = CB_WRITEEX;
            cbs.cbWriteExAll = CB_WRITEEXALL;
            cbs.cbMSGHandler = CB_MSGHandler;
            return cbs;
        }

        ///*****************************************************************************
        /// <summary>
        /// ドライバ初期化処理
        /// </summary>
        /// <param name="desc"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_INIT(string ApiName, ref UInt32 dwStatus, UInt32 dwTimeout)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                UInt32 dwApiStatus = 0;
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_INIT Start...");

                // Tデバイス固有初期化処理
                if (EGFWDESC.dyClass == null)
                {
                    //指定されたパスでライブラリがロードできない
                    EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, "Failed to load library.[" + EGFWDESC.stDLLPath + "]");
                    STATE.Error = true;//ステータスをエラーにする
                    return EDGECONST.CB_STS_FATALERR;
                }
                //初期化処理
                ret = (int)EGFWDESC.dyClass.HiInit(ApiName, ref dwApiStatus, dwTimeout);
                if(ret == EDGECONST.CB_STS_FATALERR)
                {
                    //初期化失敗
                    EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, "Failed to load library.[" + EGFWDESC.stDLLPath + "]");
                    STATE.Error = true;//ステータスをエラーにする
                    return EDGECONST.CB_STS_FATALERR;
                }
                // TODO: ↓不要の場合は削除
                EGFWDESC.cb_ConnectChk = mycb_ConnectChk;           // デバイス接続状態チェック コールバック登録
                // TODO: ↓不要の場合は削除
                EGFWDESC.cb_ReConnect = mycb_ReConnect;             // デバイス再接続処理 コールバック登録
                EGFWDESC.pDevDesc = egDevDesc;                      // デバイス固有定義情報を格納

                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_INIT End.");

                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// 通信回線をオープンし、ポーリングを開始します
        /// </summary>
        /// <param name="desc"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_OPEN(ref UInt32 dwStatus, UInt32 dwTimeout)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                UInt32 dwApiStatus = 0;

                // オープン開始
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_OPEN Start...");

                // 通信回線オープン
                if (EGFWDESC.dyClass == null) //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                    return EDGECONST.CB_STS_FATALERR;

                //オープン処理
                ret = (int)EGFWDESC.dyClass.HiOpen(ref dwApiStatus, dwTimeout);
                if (ret == EDGECONST.CB_STS_FATALERR)
                {
                    // オープン失敗
                    EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, "Open Failure. Status[{0}]", dwApiStatus.ToString("x8"));
                    STATE.Error = true;//ステータスをエラーにする
                    return EDGECONST.CB_STS_FATALERR;
                }

                // オープン成功
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_OPEN Success. Status[{0}]", ret.ToString("x8"));

                // 接続済みフラグON
                STATE.Connect = true;

                // エラーなしとする
                STATE.Error = false;

                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_OPEN End.");

                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }

        ///*****************************************************************************
        /// <summary>
        ///  通信回線をクローズします。
        /// </summary>
        /// <param name="desc"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_CLOSE(ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                UInt32 dwApiStatus = 0;

                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_CLOSE Start...");
                STATE.Connect = false;                // 接続フラグOFF
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                    return EDGECONST.CB_STS_FATALERR;

                ret = (int)EGFWDESC.dyClass.HiClose(ref dwApiStatus);                // 通信回線クローズ、リソースの解放
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_OPE, "CB_CLOSE End. Status[{0}H] API Status[{1}H]",ret.ToString("x8"),dwApiStatus.ToString("x8"));   // ログ出力

                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }


        ///*****************************************************************************
        /// <summary>
        /// デバイスからの入力値取得処理 自動
        /// [TODO]：EdgeIOポート定義に基づきデバイスからの入力処理を実装してください。
        /// </summary>
        /// <param name="usID">スレッドID</param>
        /// <param name="address">アドレス オフセット</param>
        /// <param name="pData">入力データ格納バッファ</param>
        /// <param name="readsize">入力データサイズ</param>
        /// <param name="buffsize">入力バッファサイズ</param>
        /// <param name="egio">EdgeIOポート情報</param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_READ(ushort woID, EgHgTagRef tagref, ref byte[] pData, int readsize, int buffsize, EgHgEdgeIOPort egio, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    pData = null;
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスからの入力値取得処理
                ret = (int)EGFWDESC.dyClass.HiRead(woID, egio.Offset, readsize, ref pData, buffsize, dwTimeout,  ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// デバイスからの入力処理 マニュアル
        /// [TODO]：全Tag情報 全処理を薄皮で実装します。
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_READEX(ushort usID, List<EgHgTagRef> tagRefs_IN, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスからの入力値取得処理 
                ret = (int)EGFWDESC.dyClass.HiReadEx(usID, tagRefs_IN, IndexTagRef, tagRef_IN_IOPorts, IndexIOPort, dwTimeout, ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// デバイスからの入力処理 マニュアル
        /// [TODO]：全Tag情報 全処理を薄皮で実装します。
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_READEXALL(ushort usID, List<EgHgTagRef> tagRefs_IN, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスからの入力値取得処理 
                ret = (int)EGFWDESC.dyClass.HiReadExAll(usID, tagRefs_IN, tagRef_IN_IOPorts, dwTimeout, ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// デバイスへの出力処理 自動
        /// [TODO]：1Tag情報 EdgeIOポート定義に基づきデバイスへの出力処理を実装してください。
        /// </summary>
        /// <param name="usID">スレッドID</param>
        /// <param name="address">アドレス オフセット</param>
        /// <param name="pData">出力データ</param>
        /// <param name="size">出力データサイズ</param>
        /// <param name="egio">EdgeIOポート情報</param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_WRITE(ushort usID, EgHgTagRef tagref, byte[] pData, int size, EgHgEdgeIOPort egIoPort, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    pData = null;
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスへの出力処理
                ret = (int)EGFWDESC.dyClass.HiWrite(usID, egIoPort.Offset, size, ref pData, dwTimeout, ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// デバイスへの出力処理 マニュアル
        /// [TODO]：全Tag情報 全処理を薄皮で実装します。
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_WRITEEX(ushort usID, List<EgHgTagRef> tagRefs_OUT, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts , int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスからの入力値取得処理 
                ret = (int)EGFWDESC.dyClass.HiWriteEx(usID, tagRefs_OUT, IndexTagRef, tagRef_OUT_IOPorts, IndexIOPort, dwTimeout, ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// デバイスへの出力処理 マニュアル
        /// [TODO]：全Tag情報 全処理を薄皮で実装します。
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_WRITEEXALL(ushort usID, List<EgHgTagRef> tagRefs_OUT, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, UInt32 dwTimeout, ref UInt32 dwStatus)
        {
            try
            {
                int ret = EDGECONST.CB_STS_OK;
                if (EGFWDESC.dyClass == null)             //指定されたパスでライブラリがロードできない Initでエラーの為ここでは処理を行わない
                {
                    return EDGECONST.CB_STS_FATALERR;
                }
                //デバイスからの入力値取得処理 
                ret = (int)EGFWDESC.dyClass.HiWriteExAll(usID, tagRefs_OUT, tagRef_OUT_IOPorts, dwTimeout, ref dwStatus);
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// デバイス再接続処理
        /// NOTE: コールバック登録済みの場合、通信回線障害発生時にコールされます
        /// </summary>
        /// <param name="desc"></param>
        /// <returns></returns>
        ///*****************************************************************************
        public int mycb_ReConnect(ref UInt32 dwStatus, UInt32 dwTimeout)
        {
            try
            {
                int ret = EDGECONST.CB_STS_ERROR;
                STATE.dwConnectErrorCount++;
                if (STATE.dwConnectErrorCount > STATE.dwReConnectCount)//再接続実行回数とエラー回数が一致した場合再接続して処理
                {
                    if (STATE.ExecuteReConnect == true) //再接続するフラグが立っている場合のみここに入る
                    {
                        // デバイス再接続処理関数があれば呼び出す 無ければtrue
                        if (EGFWDESC.DLLHiReConnect != null)
                            ret = (int)EGFWDESC.dyClass.HiReConnect(ref dwStatus, dwTimeout);
                        else//無い場合はクローズしてオープンを呼び出す
                        {
                            CB_CLOSE(ref dwStatus);//クローズ呼び出し エラーは無視
                            ret = CB_OPEN(ref dwStatus, dwTimeout); //オープンを呼ぶ
                        }
                    }
                }

                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// デバイス接続状態チェック
        /// NOTE: コールバック登録済みの場合、指定周期(STATE.ConnectChkCycle)でコールされます
        /// </summary>
        /// <param name="desc"></param>
        /// <returns>接続状態(true:接続済み, false:未接続)</returns>
        ///*****************************************************************************
        public bool mycb_ConnectChk(ref UInt32 dwStatus, UInt32 dwTimeout)
        {
            try
            {
                int iRet = 0;
                dwStatus = 0;		// API戻り値初期化

                bool ret = true;
                // デバイス再接続処理関数があれば呼び出す 無ければtrue
                if (EGFWDESC.DLLHiConnectChk != null)
                    ret = EGFWDESC.dyClass.HiConnectChk(ref dwStatus, dwTimeout);

                //接続がされていない場合は接続してみる
                if (ret == false)
                {
                    STATE.dwConnectErrorCount++;
                    if (STATE.dwConnectErrorCount > STATE.dwReConnectCount)//再接続実行回数とエラー回数が一致した場合再接続して処理
                    {
                        iRet = mycb_ReConnect(ref dwStatus, dwTimeout);	//再接続呼び出し
                        if (EDGECONST.CB_STS_OK == iRet)         //OKの場合再度チェックしたものを返す
                        {
                            //事前に関数があるかチェックしており無い場合は個々に入らない
                            ret = EGFWDESC.dyClass.HiConnectChk(ref dwStatus, dwTimeout);
                        }
                    }
                }
                else
                    STATE.dwConnectErrorCount = 0;//初期化
                return ret;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// Edgeメッセージハンドラ
        /// [TODO] : 必要に応じてメッセージ・処理を追加
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        ///*****************************************************************************
        public int CB_MSGHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int result = 0;
                switch (messNo)
                {
                    //
                    // 任意のメッセージ処理追加
                    // case
                    //
                    default:                        //上記以外は抜ける
                        break;
                }
                return result;
            }
            catch (Exception ex)
            {
                EGAPI.EGFW.PrintLog(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return EDGECONST.CB_STS_FATALERR;
            }
        }
    }
}
