﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Threading.Tasks;
using System.Runtime.CompilerServices;
using System.Threading;
using RTedge;   // RT-edge system API
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using ExtCommon;
using eWrap;

namespace EdgeCode
{
    /// <summary>
    /// 入出力スレッド情報
    /// </summary>
    public class IO_TRDINFO
    {
        public ushort usID;                         // スレッドID
        public SemaphoreSlim hTrigSem;              // 入出力トリガセマフォハンドル
        public CancellationTokenSource threadCTS;   // 入出力スレッドキャンセル処理用変数
        public UInt32   uiOpeCnt;                   // スレッド動作回数
        public float    flTime;                     // スレッド処理時間
        public float    flTimeMin;                  // スレッド最短処理時間
        public float    flTimeMax;                  // スレッド最長処理時間
        public IntPtr   pEntryThreadCnt;            // スレッド動作回数 のタグEntry
        public IntPtr   pEntryThreadTime;           // スレッド処理時間 のタグEntry
        public IntPtr   pEntryThreadTimeMin;        // スレッド最短処理時間 のタグEntry
        public IntPtr   pEntryThreadTimeMax;        // スレッド最長処理時間 のタグEntry
        public IntPtr   pEntryThreadLastErr;        // スレッド最新エラー値 のタグEntry
    }

    /// <summary>
    /// Edgeフレームワーク共通IF
    /// </summary>
    public class EgFWdesc
    {
        public List<IO_TRDINFO> ThreadInTbl;            // 入力スレッド情報テーブル
        public List<IO_TRDINFO> ThreadOutTbl;           // 出力スレッド情報テーブル
        public EgFwCBs.p_funcConnectChk cb_ConnectChk;  // デバイス接続チェック コールバック関数
        public EgFwCBs.p_funcReConnect cb_ReConnect;    // デバイス再接続 コールバック関数
        public EgDevDesc pDevDesc;                      // デバイス固有定義情報
        public string stDLLPath = "";                        // DLLパス
        public Assembly hDllLib = null;                 // 薄々皮 DLL (プロトコル)
        public dynamic dyClass = null;                  // クラスのインスタンス
        public Type ClassType = null;                   // type型保持
        public MethodInfo DLLHiClassInit;               // DLL HiClassInit関数
        public MethodInfo DLLHiInit;                    // DLL HiInit関数
        public MethodInfo DLLHiOpen;                    // DLL HiOpen関数
        public MethodInfo DLLHiClose;                   // DLL HiClose関数
        public MethodInfo DLLHiRead;                    // DLL HiRead関数
        public MethodInfo DLLHiWrite;                   // DLL HiWrite関数
        public MethodInfo DLLHiReadEx;                 // DLL HiReadEx関数 TagRefそのまま渡す
        public MethodInfo DLLHiWriteEx;                // DLL HiWriteEx関数 TagRefそのまま渡す
        //public MethodInfo DLLHiReadAll;                 // DLL HiReadAll関数
        //public MethodInfo DLLHiWriteAll;                // DLL HiWriteAll関数
        public MethodInfo DLLHiReConnect;               // DLL HiReConnect関数	無くてもよい
        public MethodInfo DLLHiConnectChk; 	            // DLL HiConnectChk関数 無くてもよい
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public EgFWdesc()
        {
            ThreadInTbl = new List<IO_TRDINFO>();
            ThreadOutTbl = new List<IO_TRDINFO>();
        }
    }

    /// <summary>
    /// コールバック関数定義クラス
    /// </summary>
    public class EgFwCBs
    {
        ////
        /// コールバック関数
        //
        public delegate int p_funcINIT(string ApiName, ref UInt32 dwStatus, UInt32 dwTimeout);
        public delegate int p_funcOPEN(ref UInt32 dwStatus, UInt32 dwTimeout);
        public delegate int p_funcCLOSE(ref UInt32 dwStatus);
        public delegate int p_funcREAD(ushort usID, EgHgTagRef tagref, ref byte[] Data, int size, int bufsize, EgHgEdgeIOPort egIoPort, UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate int p_funcREADEX(ushort usID, List<EgHgTagRef> tagRefs_IN, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, int IndexIOPorts, UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate int p_funcREADEXALL(ushort usID, List<EgHgTagRef> tagRefs_IN, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate int p_funcWRITE(ushort usID, EgHgTagRef tagref, byte[] Data, int size, EgHgEdgeIOPort egIoPort, UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate int p_funcWRITEEX(ushort usID, List<EgHgTagRef> tagRefs_OUT, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, int IndexIOPorts , UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate int p_funcWRITEEXALL(ushort usID, List<EgHgTagRef> tagRefs_OUT, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, UInt32 dwTimeout, ref UInt32 dwStatus);
        public delegate bool p_funcConnectChk(ref UInt32 dwStatus, UInt32 dwTimeout);
        public delegate int p_funcReConnect(ref UInt32 dwStatus, UInt32 dwTimeout);
        public delegate int p_funcMSGHandler(string senderName, int messNo, byte[] paramArray);

        public p_funcINIT       cbInit;           //デバイス初期化処理
        public p_funcOPEN       cbOpen;           //デバイスオープン処理
        public p_funcCLOSE      cbClose;          //デバイスクローズ処理
        public p_funcREAD       cbRead;           //デバイス入力処理
        public p_funcREADEX     cbReadEx;         //デバイス入力処理 薄皮で処理カスタマイズ
        public p_funcREADEXALL  cbReadExAll;      //デバイス入力処理 薄皮で処理カスタマイズ
        public p_funcWRITE      cbWrite;          //デバイス出力処理
        public p_funcWRITEEX    cbWriteEx;        //デバイス出力処理 薄皮で処理カスタマイズ
        public p_funcWRITEEXALL cbWriteExAll;     //デバイス出力処理 薄皮で処理カスタマイズ
        public p_funcConnectChk cbConnectChk;     //デバイス接続チェック
        public p_funcReConnect  cbReConnect;      //デバイス再接続
        public p_funcMSGHandler cbMSGHandler;     //個別メッセージハンドラー
    }

    /// <summary>
    /// スレッド処理時間計測クラス
    /// </summary>
    public class EgStopwatch
    {
        Stopwatch sw;
        edge_API egapi;
        MYSTATE state;
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_egapi"></param>
        /// <param name="_state"></param>
        public EgStopwatch(edge_API _egapi, MYSTATE _state)
        {
            sw = new Stopwatch();
            egapi = _egapi;
            state = _state;
        }
        /// <summary>
        /// 計測開始
        /// </summary>
        public void Start()
        {
            if (state.IsMeasure)    // 処理時間計測フラグONの場合のみ
            {
                sw.Restart();
            }
        }
        /// <summary>
        /// 計測停止
        /// </summary>
        /// <param name="trdInfo"></param>
        public void Stop(IO_TRDINFO trdInfo)
        {
            if (state.IsMeasure)    // 処理時間計測フラグONの場合のみ
            {
                sw.Stop();
                // 入出力スレッド処理時間更新
                UpdateIOThreadOpeTime(trdInfo, (float)sw.Elapsed.TotalMilliseconds);
            }
        }
        /// <summary>
        /// 入出力スレッド処理時間更新
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="time"></param>
        private void UpdateIOThreadOpeTime(IO_TRDINFO trdInfo, float time)
        {
            // ---------------------------------
            // 今回の処理時間更新
            // ---------------------------------
            trdInfo.flTime = time;
            egapi.EgTagWriteByEntry(trdInfo.pEntryThreadTime, trdInfo.flTime);

            // ---------------------------------
            // 最短処理時間更新
            // ---------------------------------
            if ((trdInfo.flTimeMin == 0)
                || ((trdInfo.flTimeMin > time) && (time != 0)))
            {
                trdInfo.flTimeMin = time;
                egapi.EgTagWriteByEntry(trdInfo.pEntryThreadTimeMin, trdInfo.flTimeMin);
            }
            // ---------------------------------
            // 最長処理時間更新
            // ---------------------------------
            if (trdInfo.flTimeMax < time)
            {
                trdInfo.flTimeMax = time;
                egapi.EgTagWriteByEntry(trdInfo.pEntryThreadTimeMax, trdInfo.flTimeMax);
            }
        }
    }

    

    /// <summary>
    /// RT-edge API/フレームワーク実装クラス
    /// </summary>
    public class edgeCode
    {
        ////
        /// edge_APIクラス インスタンス化 
        //
        public edge_API EGAPI = null;                               // edge APIのインスタンス

        public MYSTATE STATE = null;                                // サービス状態クラスのインスタンス
        public EDGECONST EGCONST = null;                            // 各種定義値
        public EgFWdesc mEgFwDesc = null;
        public string HiApiName = "";                               //ハイレベルAPIの名称 サービス名の "_"より前の文字列
        ////
        /// スレッド変数 
        //
        private CancellationTokenSource mThreadAutoTS = null;       // 自動更新スレッドキャンセル処理用変数
        private CancellationTokenSource mConnectChkTS = null;       // 通信接続監視スレッドキャンセル処理用変数

        ////
        /// メイン処理終了用コールバック関数
        //
        public delegate void CallbackMainClose();
        public CallbackMainClose mInstMainClose = null;

        ////
        /// デバイス固有処理用コールバック関数
        //
        private EgFwCBs.p_funcINIT          m_cbInit;               // デバイス初期化処理
        private EgFwCBs.p_funcOPEN          m_cbOpen;               // デバイスオープン処理
        private EgFwCBs.p_funcCLOSE         m_cbClose;              // デバイスクローズ処理
        private EgFwCBs.p_funcREAD          m_cbRead;               // デバイス入力処理 自動処理 APIを呼ぶだけ
        private EgFwCBs.p_funcREADEX        m_cbReadEx;             // デバイス入力処理 TagRef毎薄皮で処理
        private EgFwCBs.p_funcREADEXALL     m_cbReadExAll;          // デバイス入力処理 TagRef全情報を薄皮で処理
        private EgFwCBs.p_funcWRITE         m_cbWrite;              // デバイス出力処理 自動処理 APIを呼ぶだけ
        private EgFwCBs.p_funcWRITEEX       m_cbWriteEx;            // デバイス出力処理 TagRef毎薄皮で処理
        private EgFwCBs.p_funcWRITEEXALL    m_cbWriteExAll;         // デバイス出力処理 TagRef全情報を薄皮で処理
        private EgFwCBs.p_funcConnectChk    m_cbConnectChk;         // デバイス接続チェック
        private EgFwCBs.p_funcReConnect     m_cbReConnect;          // デバイス再接続
        private EgFwCBs.p_funcMSGHandler    m_cbMSGHandler;         // 個別メッセージハンドラー処理

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public edgeCode() 
        {
            EGAPI = new edge_API();
            STATE = new MYSTATE();
            EGCONST = new EDGECONST();

            mEgFwDesc = new EgFWdesc();
        }
        /// <summary>
        /// 後処理処理(デストラクタ)
        /// [TODO] : 必要に応じて変更
        /// </summary>
        public void Release()
        {
            try
            {
                // 入力スレッド終了処理
                foreach (var item in mEgFwDesc.ThreadInTbl)
                {
                    if (item.threadCTS != null)
                    {
                        item.threadCTS.Cancel();    //キャンセル状態をセットし、スレッド処理を抜ける
                        item.threadCTS = null;
                    }
                }
                // 出力スレッド終了処理
                foreach (var item in mEgFwDesc.ThreadOutTbl)
                {
                    if (item.threadCTS != null)
                    {
                        item.threadCTS.Cancel();    //キャンセル状態をセットし、スレッド処理を抜ける
                        item.threadCTS = null;
                    }
                }
                // 自動更新スレッド終了処理
                if (mThreadAutoTS != null)
                {
                    mThreadAutoTS.Cancel();         //キャンセル状態をセットし、スレッド処理で抜ける
                    mThreadAutoTS = null;
                }
                // 通信接続監視スレッド終了処理
                if (mConnectChkTS != null)
                {
                    mConnectChkTS.Cancel();         //キャンセル状態をセットし、スレッド処理を抜ける
                    mConnectChkTS = null;
                }
                // 入力スレッドトリガセマフォ
                for (int i = 0; i < mEgFwDesc.ThreadInTbl.Count; i++)
                {
                    if (mEgFwDesc.ThreadInTbl[i].hTrigSem != null)
                        mEgFwDesc.ThreadInTbl[i].hTrigSem.Dispose();
                }
                // 出力スレッドトリガセマフォ
                for (int i = 0; i < mEgFwDesc.ThreadOutTbl.Count; i++)
                {
                    if (mEgFwDesc.ThreadOutTbl[i].hTrigSem != null)
                        mEgFwDesc.ThreadOutTbl[i].hTrigSem.Dispose();
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// メソッド取得
        /// </summary>
        /// <param name="Func"></param>
        /// <param name="stFuncName"></param>
        /// <returns></returns>
        bool fnClsGetMethod(ref MethodInfo Func, string stFuncName)
        {
            if (mEgFwDesc.dyClass == null || mEgFwDesc.ClassType == null) return false; //関数呼び出し順に誤り

            Func = mEgFwDesc.ClassType.GetMethod(stFuncName);   // DLL 関数取得
            if (Func == null)//ない場合はエラー
            {
                // 指定関数のロード失敗
                LOG(edge_FW.EG_LOG_ERR, "DLL " + stFuncName + " Function Load Fiald [" + mEgFwDesc.stDLLPath + "]\n");  // ログ
                return false;
            }
            return true;
        }
        /// <summary>
        /// 薄々皮のロード
        /// </summary>
        /// <returns></returns>
        public bool fnLoadLibrary()
        {
            try
            {
                //パスの指定があるかチェック ない場合はエラー
                if (mEgFwDesc.stDLLPath.Length < 1)
                {
                    //パスの指定がない
                    LOG(edge_FW.EG_LOG_ERR, "No dll path specified.");
                    return false;
                }
                //ライブラリをロード
                mEgFwDesc.hDllLib = Assembly.LoadFrom(mEgFwDesc.stDLLPath);
                if(mEgFwDesc.hDllLib == null)
                {
                    //指定されたパスでライブラリがロードできない
                    LOG(edge_FW.EG_LOG_ERR, "Failed to load library.[" + mEgFwDesc.stDLLPath + "]");
                    return false;
                }

                //関数を取得
                Type Cls = mEgFwDesc.hDllLib.GetType(EDGECONST.EWRAP_RSL_NAME);
                if (Cls != null)
                {
                    
                    mEgFwDesc.ClassType = Cls;
                    HiApiCallConfig cfg = new HiApiCallConfig() { C_EGAPI = EGAPI, C_STATE = STATE };
                    mEgFwDesc.dyClass = Activator.CreateInstance(mEgFwDesc.ClassType);// xxxLib.CLASSNAMEを指定 
                    //mEgFwDesc.dyClass = Activator.CreateInstance(mEgFwDesc.ClassType, BindingFlags.CreateInstance, null, cfg, null);// xxxLib.CLASSNAMEを指定 
                    if (mEgFwDesc.dyClass == null)
                    {
                        //指定されたクラスがロードできない
                        LOG(edge_FW.EG_LOG_ERR, "Failed to load class.[" + EDGECONST.EWRAP_RSL_NAME + "]");
                        return false;
                    }
                    //関数のアドレスの取得HiClassInit(
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiClassInit, EDGECONST.EWRAP_RSL_CLASSINIT)) return false;
                    //mEgFwDesc.dyClass.HiClassInit(EGAPI, STATE);
                    mEgFwDesc.dyClass.HiClassInit(EGAPI, STATE.OutputDifferentFlag);

                    //関数アドレスの取得 HiInit
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiInit, EDGECONST.EWRAP_RSL_INIT)) return false;
                    //関数アドレスの取得 HiOpen
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiOpen, EDGECONST.EWRAP_RSL_OPEN)) return false;
                    //関数アドレスの取得 HiClose
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiClose, EDGECONST.EWRAP_RSL_CLOSE)) return false;
                    //関数アドレスの取得 HiRead
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiRead, EDGECONST.EWRAP_RSL_READ)) return false;
                    //関数アドレスの取得 HiWrite
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiWrite, EDGECONST.EWRAP_RSL_WRITE)) return false;
                    //関数アドレスの取得 HiReadEx
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiReadEx, EDGECONST.EWRAP_RSL_READEX)) return false;
                    //関数アドレスの取得 HiWriteEx
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiWriteEx, EDGECONST.EWRAP_RSL_WRITEEX)) return false;
                    //関数アドレスの取得 HiReConnect
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiReConnect, EDGECONST.EWRAP_RSL_RECON)) return false;
                    //関数アドレスの取得 HiConnectChk
                    if (false == fnClsGetMethod(ref mEgFwDesc.DLLHiConnectChk, EDGECONST.EWRAP_RSL_CONCHK)) return false;

                    return true;
                }
                return false;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// Edgeメッセージハンドラ
        /// [TODO] : 必要に応じてメッセージ・処理を追加
        /// </summary>
        /// <param name="senderName">メッセージ送信元</param>
        /// <param name="messNo">メッセージ番号</param>
        /// <param name="paramArray">メッセージパラメータ</param>
        /// <returns></returns>
        ///*****************************************************************************
        object WMLOCK = new object();
        public Int32 MsgHandler(string senderName, int messNo, byte[] paramArray)
        {
            try
            {
                int result = 0;
                lock (WMLOCK) //排他制御 頻繁にやり取りする場合用 不要の場合は削除
                {
                    if (m_cbMSGHandler != null)          // 個別メッセージハンドラーコールバック登録有の場合
                    {
                        result = m_cbMSGHandler(senderName, messNo, paramArray);    // 個別メッセージ処理
                    }
                    //そのまま処理は続行
                    switch (messNo) //メッセージ番号毎に処理
                    {
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_RUN:    //サービス開始指令
                            // 初期化完了済みの場合のみ、処理実行します
                            if (STATE.CheckInitEnd())
                                result = StartService();            // サービス開始実行
                            else
                                STATE.SetInitStatus(EDGECONST.INIT_STS_INITRUN);    // 初期化中(RUN受信済み)に更新 ※初期化完了後にサービス開始されます
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_STOP:   // サービス終了指令
                            {
                                if (mInstMainClose != null)
                                    mInstMainClose();               // コールバック関数からは戻りません
                                else
                                    result = KillService();
                            }
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_PAUSE:  // サービス一時停止指令
                            result = PauseService();
                            break;
                        case (int)EGDEFINE.EGMSG.EM_SERVICE_UPDATE: // サービスアップデート指令
                            switch ((int)STATE.Mode)
                            {
                                case EDGECONST.MODE_AUTO:
                                    KickInput();
                                    KickOutput();
                                    break;
                                case EDGECONST.MODE_SEMIAUTO:
                                    KickOutput();
                                    break;
                                case EDGECONST.MODE_MANUAL:
                                    KickInput();
                                    KickOutput();
                                    break;
                                default:
                                    break;
                            }
                            break;
                        //
                        // [TODO] : 任意のメッセージ処理を追加
                        //
                        //case (int)EgUser.EM_SERVICE_XXX:
                        //    break;
                        default:
                            break;
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// 接続状態監視スレッド
        /// 必要が無ければ変更不要
        /// </summary>
        /// <param name="cancelToken"></param>
        ///*****************************************************************************
        public async void ThreadConnectChk(CancellationToken cancelToken)
        {
            try
            {
                UInt32 dwStatus = 0;
                await Task.Run(() =>
                {
                    try
                    {
                        while (true)
                        {
                            if (cancelToken.IsCancellationRequested)                    // スレッドを抜ける処理
                                break;

                            if (STATE.Run && (m_cbConnectChk != null))                  // サービス実行状態、かつコールバック登録ありの場合、処理する
                            {
                                bool bConnected = m_cbConnectChk(ref dwStatus, STATE.Timeout);    // 通信接続チェック処理 コールバック
                                if (STATE.Connect != bConnected)                                //接続状態が違う場合はタグの更新
                                {
                                    fnTagWriteDrvConnect(bConnected, dwStatus);                 //接続状態タグ更新
                                }
                            }
                            System.Threading.Thread.Sleep((int)STATE.ConnectChkCycle);  // 接続監視周期待ち
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// Edgeシステム入出力自動サイクルスレッド
        /// 必要が無ければ変更不要
        /// </summary>
        ///*****************************************************************************
        public async void ThreadAutoCycle(CancellationToken cancelToken)
        {
            try
            {
				await Task.Run(() =>
                {
                    try
                    {
                        bool bFirstF = true;
                        while (true)
                        {
                            if (cancelToken.IsCancellationRequested)        // スレッドを抜ける処理
                                break;

                            fnTagWriteLive();                               // サービス実行カウンタ増加

                            if (STATE.Run)                                  // サービス実行状態なら処理する
                            {

                                if (STATE.Connect)                              // デバイス接続中のみ処理する
                                {
                                    //インターバルモード時は以下処理を行わない
                                    if (STATE.IntervalCycle == 0 || bFirstF == true)
                                    {
                                        bFirstF = false;
                                        switch (STATE.Mode)                         // サービス動作モードに応じて分岐
                                        {
                                            case EDGECONST.MODE_AUTO:               // 自動サイクルモード時（入出力共に自動更新する）
                                                KickInput();                      	// Input処理実行
                                                KickOutput();                       // Output処理実行
                                                break;
                                            case EDGECONST.MODE_SEMIAUTO:           // 半自動サイクル時（入力のみ自動更新、出力はEM_SERVICE_UPDATEで指令）
                                                KickInput();                      	// Input処理実行
                                                break;
                                            case EDGECONST.MODE_MANUAL:             // 手動サイクル時（入出力はEM_SERVICE_UPDATEで指令するので無処理）
                                                break;
                                        }
                                    }
                                }
                                else
                                    bFirstF = true;
                            }
                            else
                                bFirstF = true;


                            System.Threading.Thread.Sleep((int)STATE.Cycle);// サイクル待ち
                        }
                    }
                    catch (Exception ex)
                    {
                        LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// IN処理 Tagに値を書き込み
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnInServiceProc(IO_TRDINFO trdInfo)
        {
            //タスク処理を開始する
            try
            {
                switch(STATE.ProcMode)
                {
                    case EDGECONST.PROC_MODE_OBJECT:    //マニュアル TagRef情報を薄皮へ渡して薄皮で処理
                        fnInServiceProcObject(trdInfo);
                        break;
                    case EDGECONST.PROC_MODE_VALUE:      //自動 TagRead/TagWriteまで処理 薄皮はAPIを呼ぶだけ
                        fnInServiceProcValue(trdInfo);
                        break;
                    case EDGECONST.PROC_MODE_OBJECTS:    //マニュアル TagRefアレイ情報を薄皮へ渡して薄皮で処理
                        fnInServiceProcObjects(trdInfo);
                        break;
                    default:
                        break;
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// IN処理 マニュアル処理
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnInServiceProcObject(IO_TRDINFO trdInfo)
        {
            try
            {
                int cbRet = 0;
                int index = 0;
                int result = 0;
                UInt32 apiRet = 0;
                EgStopwatch sw = new EgStopwatch(EGAPI, STATE); //ストップウォッチ初期化
                sw.Start();                                     //ストップウォッチ開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt));  // スレッド動作回数更新
                cbRet = EDGECONST.CB_STS_OK;//ステータス初期化
                // ---------------------------------------------------------------------
                // 入力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < EGAPI.EGFW.tagRef_IN_IOPortList.Count; i++)
                {
                    index = EGAPI.EGFW.tagRef_IN_IOPortList[i].Index;//ソート前のindex
                    // デバイスからの入力 コールバック
                    result = m_cbReadEx(trdInfo.usID, EGAPI.EGFW.tagRef_IN_List, index, EGAPI.EGFW.tagRef_IN_IOPortList, i, STATE.Timeout, ref apiRet);
                    fnTagWriteInThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                    cbRet |= result;    //最終値をビットで保持

                }
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_IN);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// IN処理 マニュアル処理 全て薄皮
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnInServiceProcObjects(IO_TRDINFO trdInfo)
        {
            try
            {
                int cbRet = 0;
                UInt32 apiRet = 0;
                EgStopwatch sw = new EgStopwatch(EGAPI, STATE); //ストップウォッチ初期化
                sw.Start();                                     //ストップウォッチ開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt));  // スレッド動作回数更新
                cbRet = m_cbReadExAll(trdInfo.usID, EGAPI.EGFW.tagRef_IN_List, EGAPI.EGFW.tagRef_IN_IOPortList, STATE.Timeout, ref apiRet);
                fnTagWriteInThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_IN);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// IN処理 自動処理 値の処理まで自動
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnInServiceProcValue(IO_TRDINFO trdInfo)
        {
            //タスク処理を開始する
            try
            {
                int result = 0;
                int cbRet = 0;
                int edgeRet = 0;
                int index = 0;
                UInt32 apiRet = 0;
                EgStopwatch sw = new EgStopwatch(EGAPI, STATE);
                sw.Start();             // 処理時間計測開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt));  // スレッド動作回数更新
                cbRet = EDGECONST.CB_STS_OK;//ステータス初期化
                // ---------------------------------------------------------------------
                // 入力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < EGAPI.EGFW.tagRef_IN_IOPortList.Count; i++)
                {
                    byte[] InDataArray = new byte[8];
                    index = EGAPI.EGFW.tagRef_IN_IOPortList[i].Index;
                    if (EGAPI.EGFW.tagRef_IN_IOPortList[i].BitUse > 0)//ビット処理はまとめて実行 0以上はTRUEと判断
                    {
                        int bitDataSize = 1;//1byte取得
                        if (EGAPI.EGFW.tagRef_IN_IOPortList[i].MaxBit > 7) //ビット数が8以上(WORD単位でのbit指定の場合は2byte取得)
                            bitDataSize = 2;

                        // デバイスからの入力 コールバック
                        result = m_cbRead(trdInfo.usID, EGAPI.EGFW.tagRef_IN_List[index], ref InDataArray, bitDataSize, bitDataSize, EGAPI.EGFW.tagRef_IN_IOPortList[i], STATE.Timeout, ref apiRet);
                        fnTagWriteInThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                        if (result == EDGECONST.CB_STS_OK)
                        {
                            while (true)
                            {
                                object bitFlag = 0;
                                fnGetBitData(ref bitFlag, InDataArray, EGAPI.EGFW.tagRef_IN_IOPortList[i].BitPos);
                                // RTCDへの書込み実行
                                edgeRet = EGAPI.EgTagWriteByEntry(EGAPI.EGFW.tagRef_IN_List[index].Entry, bitFlag);
                                //次のEGIOPORTも同じオフセットの場合でかつビットデータの場合は同じデータからbit値取得するので処理継続
                                if (true == fnIsContinuousData(i, EGAPI.EGFW.tagRef_IN_IOPortList.Count, EGAPI.EGFW.tagRef_IN_IOPortList))
                                {
                                    i++;
                                    index = EGAPI.EGFW.tagRef_IN_IOPortList[i].Index; //ソート前のindex
                                }
                                else//無い場合は抜ける
                                    break;
                            }
                        }
                        cbRet |= result;    //最終値をビットで保持
                    }
                    else // ビットデータではないのでそのまま処理
                    {
                        // デバイスからの入力 コールバック
                        result = m_cbRead(trdInfo.usID, EGAPI.EGFW.tagRef_IN_List[index], ref InDataArray, EGAPI.EGFW.tagRef_IN_List[index].Size, EGAPI.EGFW.tagRef_IN_List[index].Size, EGAPI.EGFW.tagRef_IN_IOPortList[i], STATE.Timeout, ref apiRet);
                        fnTagWriteInThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                        if (result == EDGECONST.CB_STS_OK)//成功時は書き込み
                        {
                            // RTCDへの書込み実行
                            edgeRet = EGAPI.EgTagWriteByEntry(EGAPI.EGFW.tagRef_IN_List[index].Entry, InDataArray);
                        }
                        cbRet |= result;    //最終値をビットで保持
                    }
                }
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_IN);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            finally
            {
                //Releaseはトリガ処理で行うためここでは呼ばない
            }
        }
        
        /// <summary>
        /// OUT処理 Tagの値の出力
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnOutServiceProc(IO_TRDINFO trdInfo)
        {
            try
            {
                switch (STATE.ProcMode)
                {

                    case EDGECONST.PROC_MODE_OBJECT:    //マニュアル TagRef情報を薄皮へ渡して薄皮で処理
                        fnOutServiceProcObject(trdInfo);
                        break;
                    case EDGECONST.PROC_MODE_VALUE:      //自動 TagRead/TagWriteまで処理 薄皮はAPIを呼ぶだけ
                        fnOutServiceProcValue(trdInfo);
                        break;
                    case EDGECONST.PROC_MODE_OBJECTS:    //マニュアル TagRefアレイ情報を薄皮へ渡して薄皮で処理
                        fnOutServiceProcObjects(trdInfo);
                        break;
                    default:
                        break;
                }
            }
            finally
            {
                //Releaseはトリガ処理で行うためここでは呼ばない
            }
        }
        /// <summary>
        /// OUT処理 TagRef全てを薄皮へ渡して薄皮で処理
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnOutServiceProcObjects(IO_TRDINFO trdInfo)
        {
            try
            {
                int cbRet = 0;
                UInt32 apiRet = 0;

                EgStopwatch sw = new EgStopwatch(EGAPI, STATE); //ストップウォッチ初期化
                sw.Start();                                     //ストップウォッチ開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt)); // スレッド動作回数更新
                // ---------------------------------------------------------------------
                // 出力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                cbRet |= m_cbWriteExAll(trdInfo.usID, EGAPI.EGFW.tagRef_OUT_List, EGAPI.EGFW.tagRef_OUT_IOPortList, STATE.Timeout, ref apiRet);
                fnTagWriteOutThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_OUT);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// OUT処理 TagRefを薄皮へ渡して薄皮で処理
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnOutServiceProcObject(IO_TRDINFO trdInfo)
        {
            try
            {
                int cbRet = 0;
                int index = 0;
                UInt32 apiRet = 0;

                EgStopwatch sw = new EgStopwatch(EGAPI, STATE); //ストップウォッチ初期化
                sw.Start();                                     //ストップウォッチ開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt)); // スレッド動作回数更新
                cbRet = EDGECONST.CB_STS_OK;//ステータス初期化
                // ---------------------------------------------------------------------
                // 入力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < EGAPI.EGFW.tagRef_OUT_IOPortList.Count; i++)
                {
                    index = EGAPI.EGFW.tagRef_OUT_IOPortList[i].Index;//ソート前のindex
                    // デバイスへの出力 コールバック
                    cbRet |= m_cbWriteEx(trdInfo.usID, EGAPI.EGFW.tagRef_OUT_List, index, EGAPI.EGFW.tagRef_OUT_IOPortList, i, STATE.Timeout, ref apiRet);
                    fnTagWriteOutThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                }
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_OUT);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// OUT 自動処理 Tagの操作はここで、薄皮はAPIを呼ぶだけ
        /// </summary>
        /// <param name="trdInfo"></param>
        public void fnOutServiceProcValue(IO_TRDINFO trdInfo)
        {
            try
            {
                EgStopwatch sw = new EgStopwatch(EGAPI, STATE);
                int result = 0;
                int cbRet = 0;
                int index = 0;
                int bitDataSize = 1;
                UInt32 apiRet = 0;

                sw.Start();     // 処理時間計測開始
                EGAPI.EgTagWriteByEntry(trdInfo.pEntryThreadCnt, ++(trdInfo.uiOpeCnt)); // スレッド動作回数更新
                cbRet = EDGECONST.CB_STS_OK;//ステータス初期化
                // ---------------------------------------------------------------------
                // 出力タグ更新 (TagRefs_OUT) ※RTCDからの出力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < EGAPI.EGFW.tagRef_OUT_IOPortList.Count; i++)
                {
                    byte[] OutArray = new byte[8];
                    index = EGAPI.EGFW.tagRef_OUT_IOPortList[i].Index;  //ソート前のindex
                    if (EGAPI.EGFW.tagRef_OUT_IOPortList[i].BitUse > 0)//ビット処理はまとめて実行 0以上はTRUEと判断
                    {
                        bitDataSize = 1; //1byteデータ
                        if (EGAPI.EGFW.tagRef_OUT_IOPortList[i].MaxBit > 7) //ビット数が8以上(WORD単位でのbit指定の場合は2byteデータ)
                            bitDataSize = 2;
                        // デバイスからの入力 コールバック
                        result = m_cbRead(trdInfo.usID, EGAPI.EGFW.tagRef_OUT_List[index], ref OutArray, bitDataSize, bitDataSize, EGAPI.EGFW.tagRef_OUT_IOPortList[i], STATE.Timeout, ref apiRet);
                        fnTagWriteOutThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                        if (result == EDGECONST.CB_STS_OK)
                        {
                            while (true)
                            {
                                object obBitData = new object();
                                result = EGAPI.EgTagReadByEntry(EGAPI.EGFW.tagRef_OUT_List[index].Entry, ref obBitData);    //タグの値読込
                                if (result != 0)//成功時分解して書き込む
                                    break;

                                //読み込んだデータにbitを書き込む(同じオフセットの間だけ　while
                                if (false == fnSetBitData(ref OutArray, bitDataSize, obBitData, EGAPI.EGFW.tagRef_OUT_IOPortList[i].BitPos))
                                {
                                    //致命的内部エラーのため終了
                                    result = EDGECONST.CB_STS_FATALERR;
                                    break;
                                }
                                // 次の配列にデータがあり、そのデータがbitデータでかつ、同じオフセット値の場合は処理を続ける
                                if (true == fnIsContinuousData(i, EGAPI.EGFW.tagRef_OUT_IOPortList.Count, EGAPI.EGFW.tagRef_OUT_IOPortList))
                                {
                                    i++;
                                    index = EGAPI.EGFW.tagRef_OUT_IOPortList[i].Index;    //ソート前のindex
                                }
                                else//無い場合は抜ける
                                    break;
                            }
                            //値が同じ場合でも書き込むフラグの場合、または前の値と違っていたら書き込む
                            if (0 == STATE.OutputDifferentFlag || eWrapCls.fnIsEqualObj(OutArray, EGAPI.EGFW.tagRef_OUT_List[index].OldVal))
                            {
                                // デバイスへの出力 コールバック
                                cbRet |= m_cbWrite(trdInfo.usID, EGAPI.EGFW.tagRef_OUT_List[index], OutArray, bitDataSize, EGAPI.EGFW.tagRef_OUT_IOPortList[i], STATE.Timeout, ref apiRet);
                                fnTagWriteOutThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                                EGAPI.EGFW.tagRef_OUT_List[index].OldVal = OutArray;
                            }
                        }
                        cbRet |= result;
                    }
                    else //ビットデータではないのでそのまま処理
                    {
                        object OutData = OutArray;
                        // RTCDからの読み込み処理実行
                        result = EGAPI.EgTagReadByEntry(EGAPI.EGFW.tagRef_OUT_List[i].Entry, ref OutData, true);
                        if (result == 0)
                        {

                            OutArray = (byte[])OutData;
                            // デバイスへの出力 コールバック
                            cbRet |= m_cbWrite(trdInfo.usID, EGAPI.EGFW.tagRef_OUT_List[index], OutArray, EGAPI.EGFW.tagRef_OUT_List[index].Size, EGAPI.EGFW.tagRef_OUT_IOPortList[i], STATE.Timeout, ref apiRet);
                            fnTagWriteOutThreadLastErr(trdInfo.usID, apiRet);//最終エラーを書き込み
                        }
                    }
                }
                fnUpdateIdc(trdInfo, cbRet, EDGECONST.PROC_IO_MODE_OUT);//エラーインジケータ更新
                sw.Stop(trdInfo);       // 処理時間計測停止
            }
            finally
            {
                //Releaseはトリガ処理で行うためここでは呼ばない
            }
        }
        /// <summary>
        /// エラーインジケータ更新
        /// </summary>
        /// <param name="trdInfo"></param>
        /// <param name="cbRet"></param>
        /// <param name="PROC_MODE"></param>
        public void fnUpdateIdc(IO_TRDINFO trdInfo, int cbRet, int PROC_MODE)
        {
            try
            {
                UInt32 apiRet = 0;
                if ((cbRet & EDGECONST.CB_STS_FATALERR) == EDGECONST.CB_STS_FATALERR) // 致命的エラーの場合
                {
                    fnTagWriteError(true);          // エラーインジケータをtrueに更新
                    if (m_cbReConnect != null)          // デバイス再接続処理コールバック登録ありの場合
                    {
                        if (EDGECONST.CB_STS_OK == m_cbReConnect(ref apiRet, STATE.Timeout))    // デバイス再接続試行
                        {
                            fnTagWriteError(false); // 再接続OKならエラーインジケータをfalseに更新
                        }
                    }
                }
                else if (cbRet == EDGECONST.CB_STS_OK)//エラーが無い場合はステータスを0にする
                {
                    if (PROC_MODE == EDGECONST.PROC_IO_MODE_IN)
                        fnTagWriteInThreadLastErr(trdInfo.usID, 0);
                    else
                        fnTagWriteOutThreadLastErr(trdInfo.usID, 0);
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// Edgeシステムへのデータ入力
        /// </summary>
        /// <param name="cancelToken"></param>
        /// <param name="trdInfo"></param>
        ///*****************************************************************************
        public async void ThreadInService(CancellationToken cancelToken, IO_TRDINFO trdInfo)
        {
            try
            {
                if (EGAPI.EGFW.tagRef_IN_List.Count > 0 && (EGAPI.EGFW.tagRef_IN_IOPortList == null || EGAPI.EGFW.tagRef_IN_IOPortList.Count == 0))//TagRefは登録されているのにedgeI/O Portが登録されていない場合はエラー
                {
                    LOG(edge_FW.EG_LOG_ERR, "Stop IN Service. Because TagRef(TagRefs_IN) edge I/O Port is NULL.\n");
                    fnTagWriteError(true);          // エラーインジケータをtrueに更新
                    return;
                }

                EGAPI.EGFW.tagRef_IN_IOPortList.Sort(CompareAddressAsc);//降順でソート
                await Task.Run(async () =>
                {
                    
                    while (true)
                    {
                        await trdInfo.hTrigSem.WaitAsync();
                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                            break;

                        //接続している時のみ処理を行う
                        if (STATE.Connect)
                        {
                            fnInServiceProc(trdInfo);//実処理
                            if (STATE.IntervalCycle > 0)//インターバルモードの場合はここでスリープ
                            {
                                System.Threading.Thread.Sleep((int)STATE.IntervalCycle);// サイクル待ち
                                trdInfo.hTrigSem.Release();//終わったらリリースする事で実現
                            }
                        }
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// Edgeシステムからのデータ出力
        /// </summary>
        /// <param name="cancelToken"></param>
        /// <param name="trdInfo"></param>
        ///*****************************************************************************
        public async void ThreadOutService(CancellationToken cancelToken, IO_TRDINFO trdInfo)
        {
            try
            {
                if (EGAPI.EGFW.tagRef_OUT_List.Count > 0 && (EGAPI.EGFW.tagRef_OUT_IOPortList == null || EGAPI.EGFW.tagRef_OUT_IOPortList.Count == 0))//TagRefは登録されているのにedgeI/O Portが登録されていない場合はエラー
                {
                    LOG(edge_FW.EG_LOG_ERR, "Stop OUT Service. Because TagRef(TagRefs_OUT) edge I/O Port is NULL.\n");
                    fnTagWriteError(true);          // エラーインジケータをtrueに更新
                    return;
                }

                EGAPI.EGFW.tagRef_OUT_IOPortList.Sort(CompareAddressAsc);//降順でソート
                await Task.Run(async () =>
                {

                    while (true)
                    {
                        await trdInfo.hTrigSem.WaitAsync();
                        // キャンセルトークンが来たら終了
                        if (cancelToken.IsCancellationRequested)
                            break;

                        //接続している時のみ処理を行う
                        if (STATE.Connect)
                        {
                            fnOutServiceProc(trdInfo);//実処理
                            if (STATE.IntervalCycle > 0)//インターバルモードの場合はここでスリープ
                            {
                                System.Threading.Thread.Sleep((int)STATE.IntervalCycle);// サイクル待ち
                                trdInfo.hTrigSem.Release();//終わったらリリースする事で実現
                            }
                        }
                    }
                });
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// 入力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        ///*****************************************************************************
        public void KickInput()
        {
            try
            {
                for (int i = 0; i < mEgFwDesc.ThreadInTbl.Count; i++)
                {
                    mEgFwDesc.ThreadInTbl[i].hTrigSem.Release();
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// 出力サービスを1回キックする
        /// 必要が無ければ変更不要
        /// </summary>
        ///*****************************************************************************
        public void KickOutput()
        {
            try
            {
                for (int i = 0; i < mEgFwDesc.ThreadOutTbl.Count; i++)
                {
                    mEgFwDesc.ThreadOutTbl[i].hTrigSem.Release();
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービスタグ生成 ※すでに生成されている場合は無視して継続します
        /// [TODO] : 必要に応じて作成するタグを指定(タグの生成はXMLに定義し自動生成することを推奨)
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int CreateServiceTags()
        {
            try
            {
                int result = 0;
                // サービスタグ類の生成
                // [TODO] : 必要に応じて以下コードを修正
                //result |= EGAPI.EgTagCreate(EGCONST.TAG_MYSERV_XXX, (ushort)EGDEFINE.egTagDataType.UInt64, "", "sample tag");
                string strNum;
                // 入力スレッド関連タグ
                for (int i = 0; i < STATE.InThreadNum; i++)
                {
                    strNum = i.ToString("D2");
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_IN_CNT, strNum)      , (ushort)EGDEFINE.egTagDataType.UInt32,    "", "");    // 入力スレッド動作回数
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_IN_TIME, strNum)     , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 入力スレッド処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_IN_TIMEMIN, strNum)  , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 入力スレッド最短処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_IN_TIMEMAX, strNum)  , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 入力スレッド最長処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_IN_LASTERR, strNum)  , (ushort)EGDEFINE.egTagDataType.UInt32,    "", "");    // 入力スレッド最新エラー値

                }
                // 出力スレッド関連タグ
                for (int i = 0; i < STATE.OutThreadNum; i++)
                {
                    strNum = i.ToString("D2");
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_OUT_CNT, strNum)      , (ushort)EGDEFINE.egTagDataType.UInt32,    "", "");    // 出力スレッド動作回数
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_OUT_TIME, strNum)     , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 出力スレッド処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_OUT_TIMEMIN, strNum)  , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 出力スレッド最短処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_OUT_TIMEMAX, strNum)  , (ushort)EGDEFINE.egTagDataType.Float,     "", "");    // 出力スレッド最長処理時間
                    result |= EGAPI.EgTagCreateEx(string.Format(EGCONST.TAG_MYSERV_OUT_LASTERR, strNum)  , (ushort)EGDEFINE.egTagDataType.UInt32,    "", "");    // 出力スレッド最新エラー値
                }
                // その他タグ
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DRV_CONNECT, (ushort)EGDEFINE.egTagDataType.Boolean,    "", "");       // 通信接続状態
                result |= EGAPI.EgTagCreateEx(EGCONST.TAG_MYSERV_DRV_CONNECT_LASTERR, (ushort)EGDEFINE.egTagDataType.UInt32, "", "");   // 通信接続状態詳細
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// 入力スレッド情報生成
        /// </summary>
        /// <param name="usID"></param>
        /// <returns></returns>
        ///*****************************************************************************
        IO_TRDINFO CreateInThreadInfo(ushort usID)
        {
            IO_TRDINFO trdInfo = new IO_TRDINFO();
            trdInfo.usID = usID;
            trdInfo.hTrigSem = new SemaphoreSlim(1, 1);         // 入力サービススレッド用トリガセマフォ
            trdInfo.threadCTS = new CancellationTokenSource();  // スレッド終了用token指定
            string strID = usID.ToString("D2");
            object entryObj = new object();
            // タグのEntryを取得し保持
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_IN_CNT, strID), "Entry", ref entryObj);     // スレッド動作回数
            trdInfo.pEntryThreadCnt = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_IN_TIME, strID), "Entry", ref entryObj);    // スレッド処理時間
            trdInfo.pEntryThreadTime = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_IN_TIMEMIN, strID), "Entry", ref entryObj); // スレッド最短処理時間
            trdInfo.pEntryThreadTimeMin = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_IN_TIMEMAX, strID), "Entry", ref entryObj); // スレッド最長処理時間
            trdInfo.pEntryThreadTimeMax = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_IN_LASTERR, strID), "Entry", ref entryObj); // スレッドラストエラー
            trdInfo.pEntryThreadLastErr = (IntPtr)entryObj;
            return trdInfo;
        }

        ///*****************************************************************************
        /// <summary>
        /// 出力スレッド情報生成
        /// </summary>
        /// <param name="usID"></param>
        /// <returns></returns>
        ///*****************************************************************************
        IO_TRDINFO CreateOutThreadInfo(ushort usID)
        {
            IO_TRDINFO trdInfo = new IO_TRDINFO();
            trdInfo.usID = usID;
            trdInfo.hTrigSem = new SemaphoreSlim(1, 1);         // 入力サービススレッド用トリガセマフォ
            trdInfo.threadCTS = new CancellationTokenSource();  // スレッド終了用token指定
            string strID = usID.ToString("D2");
            object entryObj = new object();
            // タグのEntryを取得し保持
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_OUT_CNT, strID), "Entry", ref entryObj);     // スレッド動作回数
            trdInfo.pEntryThreadCnt = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_OUT_TIME, strID), "Entry", ref entryObj);    // スレッド処理時間
            trdInfo.pEntryThreadTime = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_OUT_TIMEMIN, strID), "Entry", ref entryObj); // スレッド最短処理時間
            trdInfo.pEntryThreadTimeMin = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_OUT_TIMEMAX, strID), "Entry", ref entryObj); // スレッド最長処理時間
            trdInfo.pEntryThreadTimeMax = (IntPtr)entryObj;
            EGAPI.EgTagGetProperty(string.Format(EGCONST.TAG_MYSERV_OUT_LASTERR, strID), "Entry", ref entryObj); // スレッドラストエラー
            trdInfo.pEntryThreadLastErr = (IntPtr)entryObj;
            return trdInfo;
        }

        ///*****************************************************************************
        /// <summary>
        /// サービスインジケータの更新 ※更新に失敗しても無視して継続します
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public int UpdateIndicator()
        {
            try
            {
                // インジケータタグはフレームワークによって自動生成済
                int result = 0;
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_STATUS, STATE.State);   // サービス起動状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_RUN, STATE.Run);        // サービス実行状態
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_LIVE, STATE.Live);      // サービス実行カウンタ
                result |= EGAPI.EgTagWrite(EGCONST.IND_MYSERV_ERROR, STATE.Error);    // サービス異常状態
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービスプロパティの取得 ※取得できない時はデフォルト設定値を採用します
        /// [TODO] : 必要に応じて設定項目の増減指定
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int GetProperty()
        {
            try
            {
                // プロパティタグはXML定義可能　無いときはデフォルト設定値を採用します
                int result = 0;
                object value = new object();
                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_MODE, ref value);    // サービス動作モードの取得
                if (result == 0)
                    STATE.Mode = (byte)value;
                else
                    STATE.Mode = EDGECONST.DEFAULT_MODE;				            // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_MODE + " " + STATE.Mode.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_CYCLE, ref value);   // サービス処理サイクルの取得
                if (result == 0)
                    STATE.Cycle = (uint)value;
                else
                    STATE.Cycle = EDGECONST.DEFAULT_CYCLE;                          // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_CYCLE + " " + STATE.Cycle.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_INPRIO, ref value);  // 入力サービススレッドのプライオリティ指定
                if (result == 0)
                    STATE.InPriority = (byte)value;
                else
                    STATE.InPriority = EDGECONST.DEFAULT_INPRIO;                    // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_INPRIO + " " + STATE.InPriority.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_OUTPRIO, ref value);  // 出力サービススレッドのプライオリティ指定
                if (result == 0)
                    STATE.OutPriority = (byte)value;
                else
                    STATE.OutPriority = EDGECONST.DEFAULT_OUTPRIO;                  // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_OUTPRIO + " " + STATE.OutPriority.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_IN_TRDNUM, ref value); // 入力サービススレッドの生成数指定
                if (result == 0)
                    STATE.InThreadNum = (byte)value;
                else
                    STATE.InThreadNum = EDGECONST.DEFAULT_IO_THREAD;                // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_IN_TRDNUM + " " + STATE.InThreadNum.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_OUT_TRDNUM, ref value);// 出力サービススレッドの生成数指定
                if (result == 0)
                    STATE.OutThreadNum = (byte)value;
                else
                    STATE.OutThreadNum = EDGECONST.DEFAULT_IO_THREAD;               // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_OUT_TRDNUM + " " + STATE.OutThreadNum.ToString());

                
                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_EXECUTE_RECONNECT, ref value);   // 接続状態監視周期の取得
                if (result == 0)
                    STATE.ExecuteReConnect = (bool)value;
                else
                    STATE.ExecuteReConnect = EDGECONST.DEFAULT_EXECUTE_RECONNECT;      // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_EXECUTE_RECONNECT + " " + STATE.ExecuteReConnect.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_CONNECTCHKCYCLE, ref value);   // 接続状態監視周期の取得
                if (result == 0)
                    STATE.ConnectChkCycle = (uint)value;
                else
                    STATE.ConnectChkCycle = EDGECONST.DEFAULT_CONNECTCHKCYCLE;      // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_CONNECTCHKCYCLE + " " + STATE.ConnectChkCycle.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_MEASURE, ref value);   // 処理時間計測フラグの取得
                if (result == 0)
                    STATE.IsMeasure = (bool)value;
                else
                    STATE.IsMeasure = false;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_MEASURE + " " + STATE.IsMeasure.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_RECONNECTCOUNT, ref value);   // 接続実行カウントの取得
                if (result == 0)
                    STATE.dwReConnectCount = (uint)value;
                else
                    STATE.dwReConnectCount = 10000;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_RECONNECTCOUNT + " " + STATE.dwReConnectCount.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_OUTPUTDIFFERENT, ref value);   // 値が違う場合のみ出力するフラグの取得
                if (result == 0)
                    STATE.OutputDifferentFlag = (byte)value;
                else
                    STATE.OutputDifferentFlag = 0;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_OUTPUTDIFFERENT + " " + STATE.OutputDifferentFlag.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_INTERVALCYCLE, ref value);   // インターバルでの処理をする際の周期(ms)
                if (result == 0)
                    STATE.IntervalCycle = (uint)value;
                else
                    STATE.IntervalCycle = 0;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_INTERVALCYCLE + " " + STATE.IntervalCycle.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_TIMEOUT, ref value);   // API呼び出し時のタイムアウト時間
                if (result == 0)
                    STATE.Timeout = (uint)value;
                else
                    STATE.Timeout = 0;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_TIMEOUT + " " + STATE.Timeout.ToString());

                result = EGAPI.EgTagRead(EGCONST.PROP_MYSERV_PROCESSMODE, ref value);   // システム用 共通処理分岐 0:TagRef送受信 1:Value送受信 2:Arrayを薄皮に渡す
                if (result == 0)
                    STATE.ProcMode = (byte)value;
                else
                    STATE.ProcMode = 0;        // デフォルト
                LOG(edge_FW.EG_LOG_DBG, EGCONST.PROP_MYSERV_PROCESSMODE + " " + STATE.ProcMode.ToString());

                //
                // [TODO] : 任意のサービスプロパティを指定
                //
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス開始 ※入出力可能になります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public int StartService()
        {
            try
            {
                UInt32 dwErrStatus = 0;
                fnTagWriteRun(true);						    // サービス実行状態に更新
                int ret = m_cbOpen(ref dwErrStatus, STATE.Timeout); // デバイスオープン処理 コールバック
                if (ret != EDGECONST.CB_STS_OK)                 // 失敗時ログ出力
                {
                    LOG(edge_FW.EG_LOG_ERR, "CB_OPEN Error. Status[" + ret.ToString() + "]");
                }
                fnTagWriteDrvConnect(STATE.Connect, dwErrStatus); // 接続状態更新
                fnTagWriteError(STATE.Error);				    // エラー状態更新

                STATE.EndStartFlag = true;                      // StartService終了フラグ
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス一時停止 ※入出力されなくなります
        /// 必要が無ければ変更不要
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int PauseService()
        {
            try
            {
                fnTagWriteRun(false);			// サービス停止状態に更新
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }

        ///*****************************************************************************
        /// <summary>
        /// サービスの初期化
        /// </summary>
        /// <returns>正常時０</returns>
        ///*****************************************************************************
        public Int32 InitService()
        {
            try
            {
                Int32 result = 0;
                UInt32 dwErrStatus = 0;

                STATE.SetInitStatus(EDGECONST.INIT_STS_INIT);           // 初期化中に更新

                //ライブラリのロード

                edge_API.EDGECONFIG config = new edge_API.EDGECONFIG(); // 設定情報の初期化
                config = EGAPI.EDGE_CONFIG_DEFAULT;                     // 初期値設定

                config.mInstMessFunc = MsgHandler;                      // Edgeメッセージユーザハンドラの指定

                result = EGAPI.EgInit(config);                          // Edgeフレームワークの初期化
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                EGCONST.SET_EDGECONST(EGAPI.EGFW.ServiceRealName);      // 各タグ名生成
                result = GetProperty();                                 // サービスプロパティのロード処理
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                result = CreateServiceTags();                           // サービスインジケータの生成
                if (result != 0)                                        // 正常終了以外の場合はコンソールにメッセージ出力
                    WARN(result);

                HiApiName = fnGetApiName(EGAPI.EGFW.ServiceRealName);   // API名取得

                if (false == fnLoadLibrary())                           // ライブラリのロード
                    return -1;

                // TODO: ★★★呼び出しタイミング確認
                int cbRet = m_cbInit(HiApiName, ref dwErrStatus, STATE.Timeout);       // デバイス初期化処理 コールバック
                if (cbRet == EDGECONST.CB_STS_OK)
                {
                    // 追加のコールバック関数登録
                    m_cbConnectChk = mEgFwDesc.cb_ConnectChk;
                    m_cbReConnect = mEgFwDesc.cb_ReConnect;
                }

                // 入力スレッド生成
                for (ushort usLp = 0; usLp < STATE.InThreadNum; usLp++)
                {
                    IO_TRDINFO trdInfo = CreateInThreadInfo(usLp);      // 入力スレッド情報生成
                    mEgFwDesc.ThreadInTbl.Add(trdInfo);                 // 入力スレッド情報テーブルへ追加
                    ThreadInService(trdInfo.threadCTS.Token, trdInfo);  // 入力スレッド処理作成
                }
                // 出力スレッド生成
                for (ushort usLp = 0; usLp < STATE.OutThreadNum; usLp++)
                {
                    IO_TRDINFO trdInfo = CreateOutThreadInfo(usLp);     // 出力スレッド情報生成
                    mEgFwDesc.ThreadOutTbl.Add(trdInfo);                // 出力スレッド情報テーブルへ追加
                    ThreadOutService(trdInfo.threadCTS.Token, trdInfo); // 出力スレッド処理作成
                }

                mThreadAutoTS = new CancellationTokenSource();          //スレッド終了用token指定
                CancellationToken cAutoToken = mThreadAutoTS.Token;
                ThreadAutoCycle(cAutoToken);                            //自動更新スレッド処理作成

                mConnectChkTS = new CancellationTokenSource();          //スレッド終了用token指定
                CancellationToken cConChkToken = mConnectChkTS.Token;
                ThreadConnectChk(cConChkToken);                         //通信接続監視スレッド処理

                fnTagWriteStatus(EDGECONST.SERVICE_UP);					// サービス起動状態更新
                
                if (STATE.CheckInitRun())                               // 初期化中(RUN受信済み)の場合
                {
                    // 初期化途中に EM_SERVICE_RUN 受信していた場合は、ここに入る
                    STATE.SetInitStatus(EDGECONST.INIT_STS_INITEND);    // 初期化完了に更新
                    StartService();                                     // サービス開始処理実行
                }
                STATE.SetInitStatus(EDGECONST.INIT_STS_INITEND);        // 初期化完了に更新

                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// サービス削除（プロセス停止）
        /// </summary>
        /// <returns></returns>
        ///*****************************************************************************
        public int KillService()
        {
            try
            {
                UInt32 dwErrStatus = 0;

                PauseService();                                         // サービス一時停止

                m_cbClose(ref dwErrStatus);                  // デバイスクローズ処理

                fnTagWriteDrvConnect(STATE.Connect, dwErrStatus);	    // 接続状態更新
                fnTagWriteError(STATE.Error);                           // エラー状態更新
                fnTagWriteStatus(EDGECONST.SERVICE_DOWN);			    // サービスダウン

                LOG(edge_FW.EG_LOG_OPE, "[LOG] End " + EGCONST.SERVICENAME + " Service");
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return -1;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        /// <param name="filePath"></param>
        /// <param name="lineNumber"></param>
        ///*****************************************************************************
        public void WARN(int reason,
                         [CallerFilePath] string filePath = "",
                         [CallerLineNumber] int lineNumber = 0)
        {
            try
            {
                Console.WriteLine(EGCONST.SERVICENAME + " service warning at file(" + filePath + ") ,line(" + lineNumber.ToString() +
                                   ")  ,reason 0x" + reason.ToString("x8"));
            }
            catch
            { }
        }
        ///*****************************************************************************
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        ///*****************************************************************************
        public void LOG(string Mode, string message)
        {
            try
            {
                Console.WriteLine(EGCONST.SERVICENAME + " : " + Mode + " : " + message);
                if(EGAPI != null && EGAPI.EGFW != null)
                    EGAPI.EGFW.PrintLog(Mode, message);
            }
            catch
            { }
        }

        ///*****************************************************************************
        /// <summary>
        /// コールバック関数の登録
        /// </summary>
        /// <param name="cbInfo"></param>
        ///*****************************************************************************
        public void AddUserFunc(EgFwCBs cbInfo)
        {
            m_cbInit    = new EgFwCBs.p_funcINIT(cbInfo.cbInit);
            m_cbOpen    = new EgFwCBs.p_funcOPEN(cbInfo.cbOpen);
            m_cbClose   = new EgFwCBs.p_funcCLOSE(cbInfo.cbClose);
            m_cbRead    = new EgFwCBs.p_funcREAD(cbInfo.cbRead);
            m_cbWrite   = new EgFwCBs.p_funcWRITE(cbInfo.cbWrite);
            m_cbReadEx = new EgFwCBs.p_funcREADEX(cbInfo.cbReadEx);
            m_cbWriteEx = new EgFwCBs.p_funcWRITEEX(cbInfo.cbWriteEx);
            m_cbReadExAll = new EgFwCBs.p_funcREADEXALL(cbInfo.cbReadExAll);
            m_cbWriteExAll = new EgFwCBs.p_funcWRITEEXALL(cbInfo.cbWriteExAll);
            m_cbMSGHandler = new EgFwCBs.p_funcMSGHandler(cbInfo.cbMSGHandler);
        }

        ////-----------------------------------------------------------------------------
        /// その他関数群
        //-------------------------------------------------------------------------------
        ///*****************************************************************************
        /// サービス名からAPI名を取得
        ///*****************************************************************************
        public string fnGetApiName(string ServiceRealName)
        {
            string ApiName = ServiceRealName;
            try
            {
                if(ServiceRealName.Contains(EDGECONST.SERVICENAME_DELIMITER))
                    ApiName = ServiceRealName.Substring(0, ServiceRealName.IndexOf(EDGECONST.SERVICENAME_DELIMITER));
                return ApiName;
            }
            catch
            {
                return ApiName;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// ビットデータの取得
        /// </summary>
        ///*****************************************************************************
        public void fnGetBitData(ref object dst, object src, ushort bit)
        {
            try
            {
                Int32 tempValue = Convert.ToInt32(src);//4バイトにとりあえず変換 //object型ではビット操作ができないため

                int flag = tempValue & (1 << bit);
                if (flag == 0)  //ビットが立っていない場合はfalse(0)を設定
                    dst = 0;
                else
                    dst = 1;    //ビットが立っているので1を設定
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// ビットデータの設定
        /// </summary>
        ///*****************************************************************************
        bool fnSetBitData(ref byte[] dst, int dstsize, object src, ushort bit)
        {
            try
            {
                // パラメータのアドレス指定が妥当か
                if (dstsize == 0)			return false;

	            if (bit > 7 && dstsize< 2) //2バイト以上のビット指定にも拘わらずdstサイズが正しくない場合はエラー
		            return false;

                int flag = 0;  //オブジェクト型からの変換
                flag = Convert.ToUInt16(src); //とりあえず2バイトに変換


                Int32 tempValue = Convert.ToInt32(dst);//4バイトにとりあえず変換 //object型ではビット操作ができないため

                if (flag == 0)//0は落とす
                    tempValue &= ~(1 << bit);     // 所定ビットを落とす
                else
                    tempValue |= (1 << bit);     // 所定ビット立てる

                //型を元に戻す
                dst = BitConverter.GetBytes(tempValue);
                //Convert.ChangeType(tempValue, Convert.GetTypeCode(dst));

                return true;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        ///*****************************************************************************
        /// <summary>
        /// bitのかたまりで処理を継続するか否かをチェックする TRUE:継続 FALSE:終了
        /// </summary>
        ///*****************************************************************************
        public bool fnIsContinuousData(int index, int RefSize, List<EgHgEdgeIOPort> PortList)
        {
            //次のEGIOPORTも同じオフセットの場合でかつビットデータの場合は同じデータからbit値取得するので処理継続
            if (index + 1 < RefSize &&
                PortList[index].Offset == PortList[index + 1].Offset &&
                PortList[index].BitUse > 0)
                return true;
            return false;
        }
        ///*****************************************************************************
        /// <summary>
        /// ソート関数 降順 小さい順
        /// </summary>
        /// <param name="a"></param>
        /// <param name="b"></param>
        ///*****************************************************************************
        public static int CompareAddressAsc(EgHgEdgeIOPort a, EgHgEdgeIOPort b)
        {
            // aとbの比較  オフセット
            if (a.Offset > b.Offset)//aの方が大きい　aがbより後になる
                return 1;
            else if (a.Offset < b.Offset)//bの方が大きい aがbより前になる
                return -1;

            //Offsetが一緒の場合はビットで比較
            if (a.BitPos > b.BitPos)    //aがbより後になる
                return 1;
            else if (a.BitPos < b.BitPos)//aがbより前になる
                return -1;

            return 0;
        }
        /// <summary>
        /// サービス起動状態を書き込む
        /// </summary>
        /// <param name="byStatus">変更後のStatus</param>
        void fnTagWriteStatus(byte byStatus)
        {
            // 変化がない場合は、処理無効
            if (STATE.State == byStatus) { return; }
            // サービス起動状態の変化をログに出力する
            LOG(edge_FW.EG_LOG_DBG, EGCONST.IND_MYSERV_STATUS + " Change. " + STATE.State.ToString() + " > " + byStatus .ToString());

            // サービス起動状態の更新
            STATE.State = byStatus;
            EGAPI.EgTagWrite(EGCONST.IND_MYSERV_STATUS, STATE.State);   // サービス起動状態
        }
        /// <summary>
        /// サービス実行中を示すフラグを書き込む
        /// </summary>
        /// <param name="boRun"></param>
        void fnTagWriteRun(bool boRun)
        {
            STATE.Run = boRun;
            EGAPI.EgTagWrite(EGCONST.IND_MYSERV_RUN, STATE.Run);   // サービス実行中フラグ
        }
        /// <summary>
        /// サービス実行中を示すインクリメントをタグに書き込む
        /// </summary>
        void fnTagWriteLive()
        {
            STATE.Live++;
            EGAPI.EgTagWrite(EGCONST.IND_MYSERV_LIVE, STATE.Live);   // サービス実行中を示すインクリメントタグ
        }
        /// <summary>
        /// サービス実行障害発生中フラグのON/OFFをタグに書き込む
        /// </summary>
        /// <param name="boErr"></param>
        void fnTagWriteError(bool boErr)
        {

            STATE.Error = boErr;
            EGAPI.EgTagWrite(EGCONST.IND_MYSERV_ERROR, STATE.Error);   // サービス実行障害発生中フラグ
            if (STATE.Error != boErr)   //変化がある時のみ出力
                LOG(edge_FW.EG_LOG_DBG, "Error Status [" + STATE.Error.ToString() + "]");//エラーはログを出力
        }
        /// <summary>
        /// 通信接続状態フラグをタグに書き込む
        /// </summary>
        /// <param name="boConnect"></param>
        /// <param name="errStatus"></param>
        void fnTagWriteDrvConnect(bool boConnect, UInt32 errStatus)
        {
            STATE.Connect = boConnect;
            EGAPI.EgTagWrite(EGCONST.TAG_MYSERV_DRV_CONNECT, STATE.Connect);   // 通信接続状態フラグ
            EGAPI.EgTagWrite(EGCONST.TAG_MYSERV_DRV_CONNECT_LASTERR, errStatus);    //エラー値を書き込む
            if (STATE.Connect != boConnect) //変化がある時のみ出力
                LOG(edge_FW.EG_LOG_DBG, "Connect Status [" + errStatus.ToString("x8") + "H]");//エラーはログを出力
        }
        /// <summary>
        /// 入力スレッド最新エラー値を書き込む
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="errStatus"></param>
        void fnTagWriteInThreadLastErr(UInt16 woID, UInt32 errStatus)
        {
            //ParamCheck
            if ((woID < 0) || (woID > EDGECONST.MAX_IO_THREAD)) { return; }
            EGAPI.EgTagWriteByEntry(mEgFwDesc.ThreadInTbl[woID].pEntryThreadLastErr, errStatus);
        }
        /// <summary>
        /// 出力スレッド最新エラー値を書き込む
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="errStatus"></param>
        void fnTagWriteOutThreadLastErr(UInt16 woID, UInt32 errStatus)
        {
            //ParamCheck
            if ((woID < 0) || (woID > EDGECONST.MAX_IO_THREAD)) { return; }
            EGAPI.EgTagWriteByEntry(mEgFwDesc.ThreadOutTbl[woID].pEntryThreadLastErr, errStatus);
        }
        
    }
}
