﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
namespace ExtCommon
{
    /// <summary>
    /// RT-edge API/フレームワーク 各種値定義クラス
    /// </summary>
    public class EDGECONST
    {
        /// <summary>
        /// 各種定義追加
        /// </summary>

        public const string TAG_PREFIX = "SERVICE.";            // サービスプロパティタグ名先頭に付与する文字列
        public const byte SERVICE_UP = 1;                       // 起動中
        public const byte SERVICE_DOWN = 0;                     // 停止中
        public const int MODE_AUTO = 1;                         // 動作モード 入出自動サイクル
        public const int MODE_SEMIAUTO = 2;                     // 動作モード 入力サイクル＋出力デマンド
        public const int MODE_MANUAL = 3;                       // 動作モード 入出力デマンド
        public const int DEFAULT_MODE = MODE_SEMIAUTO;          // 動作モード デフォルト値
        public const int DEFAULT_CYCLE = 1000;                  // 動作サイクル デフォルト値
        public const int DEFAULT_INPRIO = 148;                  // Edgeシステムへの入力サービススレッドプライオリティ デフォルト値
        public const int DEFAULT_OUTPRIO = 149;                 // Edgeシステムからの出力サービススレッドプライオリティ デフォルト値
        public const int DEFAULT_IO_THREAD = 32;                // Edgeシステムの入出力サービススレッド生成数 デフォルト値
        public const int DEFAULT_CONNECTCHKCYCLE = 1000;        // 接続状態監視周期 デフォルト値
        public const bool DEFAULT_EXECUTE_RECONNECT = false;    // 再接続するか否かのフラグ デフォルト値 再接続しない

        public const int CB_STS_OK = 0;                         // コールバック処理結果 正常
        public const int CB_STS_ERROR = 0x00000001;       // コールバック処理結果 エラー
        public const int CB_STS_FATALERR = 0x00000010;       // コールバック処理結果 致命的エラー
        public const int INIT_STS_IDLE = 0;                     // サービス未初期化
        public const int INIT_STS_INIT = 1;                     // サービス初期化中
        public const int INIT_STS_INITRUN = 2;                  // サービス初期化中(RUN受信済み)
        public const int INIT_STS_INITEND = 3;                  // サービス初期化完了

        public const int MAX_IO_THREAD = 32;                    // Thread生成数最大値

        public const int PROC_MODE_OBJECT = 0;                  // 共通処理 オブジェクトを関数へ渡す(Tagへの書き込み・読み込みは行わない。)
        public const int PROC_MODE_VALUE = 1;                   // 共通処理 値の入出力(Tagへの書き込み・読み込みは自動)
        public const int PROC_MODE_OBJECTS = 2;                 // 共通処理 オブジェクト全てを関数へ渡す(Tagへの書き込み・読み込みは行わない。)

        public const int PROC_IO_MODE_IN = 0;                   // 処理分岐用モード IN処理
        public const int PROC_IO_MODE_OUT = 1;                  // 処理分岐用モード OUT処理

        ////
        /// サービス名
        //
        public string SERVICENAME;                              // サービス名 ※ECIファイル(XML)から取得
        ////
        /// サービスインジケータ名
        //        
        public string IND_MYSERV_STATUS;                        // サービス起動
        public string IND_MYSERV_ERROR;                         // サービスエラー発生中
        public string IND_MYSERV_RUN;		                    // サービス実行中
        public string IND_MYSERV_LIVE;                          // サービス実行カウンタ
        ////
        /// サービスプロパティタグ名
        //        
        public string PROP_MYSERV_MODE;                         // サービス動作モード
        public string PROP_MYSERV_CYCLE;                        // サービス動作サイクル(ms)
        public string PROP_MYSERV_INPRIO;                       // Edgeシステムへの入力サービススレッドプライオリティ
        public string PROP_MYSERV_OUTPRIO;                      // Edgeシステムからの出力サービススレッドプライオリティ
        public string PROP_MYSERV_IN_TRDNUM;                    // Edgeシステムへの入力サービススレッド生成数
        public string PROP_MYSERV_OUT_TRDNUM;                   // Edgeシステムからの出力サービススレッド生成数
        public string PROP_MYSERV_EXECUTE_RECONNECT;            // 失敗時再接続するか否かのフラグ
        public string PROP_MYSERV_CONNECTCHKCYCLE;              // 接続状態監視周期(ms)
        public string PROP_MYSERV_MEASURE;                      // 処理時間計測フラグ
        public string PROP_MYSERV_RECONNECTCOUNT;               // 再接続実行カウント
        public string PROP_MYSERV_OUTPUTDIFFERENT;              // 値が違う時のみ出力するフラグ
        public string PROP_MYSERV_INTERVALCYCLE;                // サイクル周期での処理ではなくインターバルでの処理でのサイクル(ms)
        public string PROP_MYSERV_TIMEOUT;                      // API呼び出し時のタイムアウト デフォルト0     
        public string PROP_MYSERV_PROCESSMODE;                  // システム用 共通処理分岐 0:TagRef送受信 1:Value送受信 2:Arrayを薄皮に渡す
        ////
        /// サービス提供タグ名
        //  [TODO] : 任意のタグ追加 (XMLで定義することを推奨)
        // public static string TAG_MYSERV_XXX = "XXX";
        public string TAG_MYSERV_IN_CNT;                        // 入力スレッド動作回数
        public string TAG_MYSERV_IN_TIME;                       // 入力スレッド処理時間
        public string TAG_MYSERV_IN_TIMEMIN;                    // 入力スレッド最短処理時間
        public string TAG_MYSERV_IN_TIMEMAX;                    // 入力スレッド最長処理時間
        public string TAG_MYSERV_IN_LASTERR;                    // 入力スレッド最新エラー値
        public string TAG_MYSERV_OUT_CNT;                       // 出力スレッド動作回数
        public string TAG_MYSERV_OUT_TIME;                      // 出力スレッド処理時間
        public string TAG_MYSERV_OUT_TIMEMIN;                   // 出力スレッド最短処理時間
        public string TAG_MYSERV_OUT_TIMEMAX;                   // 出力スレッド最長処理時間
        public string TAG_MYSERV_OUT_LASTERR;                   // 出力スレッド最新エラー値

        public string TAG_MYSERV_DRV_CONNECT;                   // 通信接続状態
        public string TAG_MYSERV_DRV_CONNECT_LASTERR;           // 通信接続状態エラー詳細

        public const string EWRAP_RSL_NAME = "eWrap.HiApiCall"; //薄々皮 ネームスペース＋クラス名

        public const string EWRAP_RSL_CLASSINIT = "HiClassInit";//薄々皮 クラス初期化関数 
        public const string EWRAP_RSL_INIT = "HiInit";          //薄々皮 初期化関数 
        public const string EWRAP_RSL_OPEN = "HiOpen";          //薄々皮 Open関数
        public const string EWRAP_RSL_CLOSE = "HiClose";        //薄々皮 Close関数
        public const string EWRAP_RSL_READ = "HiRead";          //薄々皮 読込関数
        public const string EWRAP_RSL_WRITE = "HiWrite";        //薄々皮 書込関数
        public const string EWRAP_RSL_READEX = "HiReadEx";      //薄々皮 読込関数Tag指定
        public const string EWRAP_RSL_WRITEEX = "HiWriteEx";    //薄々皮 書込関数Tag指定
        public const string EWRAP_RSL_RECON = "HiReConnect";    //薄々皮 再接続関数
        public const string EWRAP_RSL_CONCHK = "HiConnectChk";  //薄々皮 接続チェック関数

        ////
        /// その他定義
        //
        public const string SERVICENAME_DELIMITER = "_";            // サービス名複数インスタンス時の区切り文字
        ////
        /// サービス固有メッセージ
        //  [TODO] : 任意のメッセージID追加(ユーザメッセージ 20000～29999番指定可)
        // public static int EM_SERVICE_XXX = 20000;    //任意の番号
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="Name"></param>
        public EDGECONST()
        { }

        /// <summary>
        /// サービス名から各タグ名称を生成
        /// </summary>
        /// <param name="Name"></param>
        public void SET_EDGECONST(string Name)
        {
            // NOTE: サービス名から各タグ名称を生成します。
            SERVICENAME = Name;
            IND_MYSERV_STATUS = TAG_PREFIX + SERVICENAME + ".Status";           // サービス起動
            IND_MYSERV_ERROR = TAG_PREFIX + SERVICENAME + ".Error";             // サービスエラー発生中
            IND_MYSERV_RUN = TAG_PREFIX + SERVICENAME + ".Run";                 // サービス実行中
            IND_MYSERV_LIVE = TAG_PREFIX + SERVICENAME + ".Live";               // サービス実行カウンタ

            PROP_MYSERV_MODE = TAG_PREFIX + SERVICENAME + ".Mode";              // サービス動作モード
            PROP_MYSERV_CYCLE = TAG_PREFIX + SERVICENAME + ".Cycle";            // サービス動作周期(ms)
            PROP_MYSERV_INPRIO = TAG_PREFIX + SERVICENAME + ".InPriority";      // Edgeシステムへの入力サービススレッドプライオリティ
            PROP_MYSERV_OUTPRIO = TAG_PREFIX + SERVICENAME + ".OutPriority";    // Edgeシステムからの出力サービススレッドプライオリティ	
            PROP_MYSERV_IN_TRDNUM = TAG_PREFIX + SERVICENAME + ".InThreadNum";  // Edgeシステムへの入力サービススレッド生成数
            PROP_MYSERV_OUT_TRDNUM = TAG_PREFIX + SERVICENAME + ".OutThreadNum";// Edgeシステムからの出力サービススレッド生成数
            PROP_MYSERV_CONNECTCHKCYCLE = TAG_PREFIX + SERVICENAME + ".ConnectChkCycle";    // 接続状態監視周期(ms)
            PROP_MYSERV_MEASURE = TAG_PREFIX + SERVICENAME + ".Measure";        // 処理時間計測フラグ
            PROP_MYSERV_RECONNECTCOUNT = TAG_PREFIX + SERVICENAME + ".ReConnectCount"; // 再接続実行カウント
            PROP_MYSERV_OUTPUTDIFFERENT = TAG_PREFIX + SERVICENAME + ".OutputDifferent"; // 値が違う時のみ出力するフラグ

            PROP_MYSERV_INTERVALCYCLE = TAG_PREFIX + SERVICENAME + ".IntervalCycle"; // サイクル周期での処理ではなくインターバルでの処理に使用する周期(ms) 0 はインターバルを使用しない
            PROP_MYSERV_TIMEOUT = TAG_PREFIX + SERVICENAME + ".Timeout";            // API呼び出し時のタイムアウト時間
            PROP_MYSERV_PROCESSMODE = TAG_PREFIX + SERVICENAME + ".ProcessMode";    // システム用 共通処理分岐 0:TagRef送受信 1:Value送受信

            TAG_MYSERV_IN_CNT = TAG_PREFIX + SERVICENAME + ".InThread{0}_Cnt";        // 入力スレッド動作回数
            TAG_MYSERV_IN_TIME = TAG_PREFIX + SERVICENAME + ".InThread{0}_Time";       // 入力スレッド処理時間
            TAG_MYSERV_IN_TIMEMIN = TAG_PREFIX + SERVICENAME + ".InThread{0}_TimeMin";    // 入力スレッド最短処理時間
            TAG_MYSERV_IN_TIMEMAX = TAG_PREFIX + SERVICENAME + ".InThread{0}_TimeMax";    // 入力スレッド最長処理時間
            TAG_MYSERV_IN_LASTERR = TAG_PREFIX + SERVICENAME + ".InThread{0}_LastError";  // 入力スレッド最新エラー値
            TAG_MYSERV_OUT_CNT = TAG_PREFIX + SERVICENAME + ".OutThread{0}_Cnt";       // 出力スレッド動作回数
            TAG_MYSERV_OUT_TIME = TAG_PREFIX + SERVICENAME + ".OutThread{0}_Time";      // 出力スレッド処理時間
            TAG_MYSERV_OUT_TIMEMIN = TAG_PREFIX + SERVICENAME + ".OutThread{0}_TimeMin";   // 出力スレッド最短処理時間
            TAG_MYSERV_OUT_TIMEMAX = TAG_PREFIX + SERVICENAME + ".OutThread{0}_TimeMax";   // 出力スレッド最長処理時間
            TAG_MYSERV_OUT_LASTERR = TAG_PREFIX + SERVICENAME + ".OutThread{0}_LastError"; // 出力スレッド最新エラー値

            TAG_MYSERV_DRV_CONNECT = TAG_PREFIX + SERVICENAME + ".Connected";               // 通信接続状態
            TAG_MYSERV_DRV_CONNECT_LASTERR = TAG_PREFIX + SERVICENAME + ".ConnectLastError";// 通信接続状態詳細

            PROP_MYSERV_EXECUTE_RECONNECT = TAG_PREFIX + SERVICENAME + ".ExecuteReConnect"; // 失敗時再接続するか否かのフラグ
        }

    }
    ////
    /// サービス内部状態
    //
    public class MYSTATE
    {

        //変数
        public byte InitState;              // サービス初期化状態(0:未初期化, 1:初期化中, 2:初期化中(Run受信済み), 3:初期化完了)
        public byte State;                  // インジケータ.エッジシステム起動状態
        public bool Run;                    // インジケータ.サービス実行状態
        public bool Error;                  // インジケータ.サービス異常状態
        public UInt32 Live;                 // インジケータ.サービス実行カウンタ
        public UInt64 StartTime;            // インジケータ.サービス開始時のタイムスタンプ
        public byte Mode;                   // プロパティ.サービス実行モード
        public UInt32 Cycle;                // プロパティ.サービス実行サイクル(ms)
        public byte InPriority;             // プロパティ.サービス(入力)プライオリティ
        public byte OutPriority;            // プロパティ.サービス(出力)プライオリティ
        public byte InThreadNum;            // プロパティ.サービス(入力)スレッド生成数
        public byte OutThreadNum;           // プロパティ.サービス(出力)スレッド生成数
        public bool Connect;                // インジケータ.通信接続状態
        public UInt32 ConnectChkCycle;      // プロパティ.通信接続監視周期(ms)
        public bool IsMeasure;              // プロパティ.スレッド処理時間計測フラグ
        public UInt32 dwReConnectCount;     // プロパティ.再接続実行カウント
        public byte OutputDifferentFlag;    // プロパティ.値が違う場合のみ出力するフラグ
        public UInt32 dwConnectErrorCount;  // インジケータ.接続エラーカウント
        public UInt32 IntervalCycle;        // プロパティ.周期処理ではなくインターバルでの周期で実行する際のサイクル(ms) 0の時は通常周期
        public UInt32 Timeout;              // プロパティ.API呼び出し時のタイムアウト時間
        public bool EndStartFlag;           // 内部フラグ.Start Service関数が一度でも終了したかどうかのフラグ	
        public bool bIsOutProcWorking;      // 内部フラグ.Out処理が動作しているか否か Intervalモード時に使用	
        public bool bIsInProcWorking;       // 内部フラグ.In処理が動作しているか否か Intervalモード時に使用	
        public byte ProcMode;                // 内部フラグ.共通処理分岐 0:TagRef送受信 1:Value送受信
        public bool ExecuteReConnect;       // プロパティ.再接続するか否かのフラグ
        //[TODO] : 任意のステータス追加
        public MYSTATE()
        {
            InitState = EDGECONST.INIT_STS_IDLE;
            State = EDGECONST.SERVICE_DOWN;
            Run = false;
            Error = false;
            Live = 0;
            StartTime = 0;
            Mode = 0;
            Cycle = 1000;
            InPriority = 149;
            OutPriority = 148;
            InThreadNum = 0;
            OutThreadNum = 0;
            Connect = false;
            ConnectChkCycle = 1000;
            IsMeasure = false;
            EndStartFlag = false;
            dwReConnectCount = 0;
            OutputDifferentFlag = 0;
            dwConnectErrorCount = 0;
            IntervalCycle = 0;
            Timeout = 0;
            bIsOutProcWorking = false;
            bIsInProcWorking = false;
            ProcMode = 0;
        }
        /// <summary>
        /// 初期化完了済みか？(true:済み,false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitEnd()
        {
            return (InitState == EDGECONST.INIT_STS_INITEND);
        }
        /// <summary>
        /// 初期化中(RUN受信済み)か？(true:初期化中(RUN受信済み),false:未)
        /// </summary>
        /// <returns></returns>
        public bool CheckInitRun()
        {
            return (InitState == EDGECONST.INIT_STS_INITRUN);
        }
        /// <summary>
        /// 初期化状態更新
        /// </summary>
        /// <param name="sts"></param>
        public void SetInitStatus(byte sts)
        {
            InitState = sts;
        }
    }
}
