﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Reflection;
using RTedge;   // RT-edge system API
using ExtCommon;
namespace eWrap
{
            /// <summary>
        /// 薄皮初期設定に渡す構造体
        /// </summary>
        public struct HiApiCallConfig
        {
            public edge_API C_EGAPI;
            public MYSTATE C_STATE;
        }
    /// <summary>
    /// 薄皮処理 HiAPI呼び出し共通クラス
    /// </summary>
    public class eWrapCls
    {
        public const UInt32 CB_STS_OK = 0;                      // 正常終了
        public const UInt32 CB_STS_ERROR = 0x00000001;          // 復旧可能エラー
        public const UInt32 CB_STS_FATALERR = 0x00000010;       // 致命的エラー
        public const UInt32 EW_RTCD_RECORD_NUM_ERROR = 0xFFFFFFF6;      // レコード数が制限値をオーバーしています
        public const UInt32 EW_RTCD_BUFFER_FULL_ERROR = 0xFFFFFFF7;      // バッファフルエラー
        public const UInt32 EW_RTCD_EXIST_ERROR = 0xFFFFFFF8;      // 同名MBX/共有メモリ存在
        public const UInt32 EW_TTABLE_ISEXIST_TAGNAME = 0xFFFFFFF9;      // 既に同じ名前のオブジェクトが登録済みです
        public const UInt32 EW_TTABLE_NOEXISTT_DSETNAME = 0xFFFFFFFA;      // 指定されたEgDataset名は登録されていません
        public const UInt32 EW_RTCD_ALREADY_OPEN_READ_ERROR = 0xFFFFFFFB;      // 既に読込オープンされています
        public const UInt32 EW_DATASET_NOTFOUND = 0xFFFFFFFC;   // 指定されたデータセットが定義されていません。またはタグが登録されていません。
        public const UInt32 EW_ILLEGALDATA = 0xFFFFFFFD;        // データが不正です
        public const UInt32 EW_NOADDRESS = 0xFFFFFFFE;          // Addressに定義が無い
        public const UInt32 EW_NOAPI = 0xFFFFFFFF;              // 関数が無い
       
        public string HIAPI_RSI_DLLNAME = "HiTCP.dll";      // DLL名
        //public const string HIAPI_RSI_DLLNAME = "rsi.dll";      // DLL名
        public string HIAPI_RSI_CLASSNAME = "HiTCP";        // クラス名
        public Assembly hRsiDllLib = null;                      // HiAPI Library DLL (プロトコル)
        public dynamic dyRsiClass = null;                       // HiAPI クラスのインスタンス
        public Type ClassType = null;                           // HiAPI クラスのタイプ
        public const string HIAPI_RSL_FUNCNAME_INIT = "Init";   // HiAPI クラスの関数名
        public const string HIAPI_RSL_FUNCNAME_OPEN = "Open";   // HiAPI クラスの関数名
        public const string HIAPI_RSL_FUNCNAME_CLOSE = "Close"; // HiAPI クラスの関数名
        public const string HIAPI_RSL_FUNCNAME_READ = "Read";   // HiAPI クラスの関数名
        public const string HIAPI_RSL_FUNCNAME_WRITE = "Write"; // HiAPI クラスの関数名
        public const string HIAPI_CONFIG_DELIMITER = "|";       // 設定文字列の区切り文字

        public byte OutputDifferentFlag = 0;
        public edge_API EGAPI;
        public eWrapCls()
        {
        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public eWrapCls(edge_API _EGAPI)
        {
            EGAPI = _EGAPI;
        }

        /// <summary>
        /// RSIのロード
        /// </summary>
        /// <returns></returns>
        public bool fnLoadLibrary()
        {
            try
            {
                //ライブラリをロード
                hRsiDllLib = Assembly.LoadFrom(HIAPI_RSI_DLLNAME);
                if (hRsiDllLib == null)
                {
                    //指定されたパスでライブラリがロードできない
                    LOG(edge_FW.EG_LOG_ERR, "Failed to load library.[" + HIAPI_RSI_DLLNAME + "]");
                    return false;
                }
                //関数を取得
                //
                string nsp_cls_name = "";
                foreach (Type type in hRsiDllLib.GetTypes())
                {
                    nsp_cls_name = type.FullName;
                    if (nsp_cls_name.Contains(".") == true)
                    {
                        string classname = nsp_cls_name.Substring(nsp_cls_name.LastIndexOf(".") + 1);
                        if (classname == HIAPI_RSI_CLASSNAME)
                            break;
                        nsp_cls_name = "";
                    }

                }
                if (nsp_cls_name.Length == 0) return false;

                ClassType = hRsiDllLib.GetType(nsp_cls_name);
                if (ClassType == null)
                {
                    //指定されたクラスがロードできない
                    LOG(edge_FW.EG_LOG_ERR, "Failed to load class.[" + dyRsiClass + "]");
                    return false;
                }
                dyRsiClass = Activator.CreateInstance(ClassType);// Rsi.CLASSNAMEを指定 
                if (dyRsiClass == null)
                {
                    //指定されたクラスがロードできない
                    LOG(edge_FW.EG_LOG_ERR, "Failed to load class.[" + dyRsiClass + "]");
                    return false;
                }

                //関数アドレスのチェック Init
                if (false == fnClsChkMethod(HIAPI_RSL_FUNCNAME_INIT)) return false;
                //関数アドレスのチェック Open
                if (false == fnClsChkMethod(HIAPI_RSL_FUNCNAME_OPEN)) return false;
                //関数アドレスのチェック Close
                if (false == fnClsChkMethod(HIAPI_RSL_FUNCNAME_CLOSE)) return false;
                //関数アドレスのチェック Read
                if (false == fnClsChkMethod(HIAPI_RSL_FUNCNAME_READ)) return false;
                //関数アドレスのチェック Write
                if (false == fnClsChkMethod(HIAPI_RSL_FUNCNAME_WRITE)) return false;

                return true;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return false;
            }
        }
        /// <summary>
        /// HiInit クラス初期化
        /// </summary>
        /// <param name="dwAPIStatus">APIのステータスを戻す</param>
        /// <returns>ステータス</returns>
        
        public virtual void HiClassInit(object a, object b)
        //public virtual void HiClassInit(edge_API _EGAPI, MYSTATE _State)
        {
            try
            {
                
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// HiInit ドライバ初期化処理
        /// </summary>
        /// <param name="dwAPIStatus">APIのステータスを戻す</param>
        /// <returns>ステータス</returns>
        public virtual UInt32 HiInit(string APIName, ref UInt32 dwAPIStatus, UInt32 dwTimeout)
        {
            try
            {
                //ライブラリのロード
                if (fnLoadLibrary() == false) return CB_STS_FATALERR; //内部でエラー
                
                string config = "";         //設定値

                fnServiceTagNameCreate();   //固有Tag作成
                fnSetConfig(ref config);    //タグから設定値を読み込みconfigの設定
                
                //init実行
                dwAPIStatus = dyRsiClass.Init(APIName, config, dwTimeout);   //APIコール
                LOG(edge_FW.EG_LOG_DBG, "End HiInit config[" + config + "] Status[" + dwAPIStatus.ToString("x8") + "H]");
                return fnRetValue(dwAPIStatus);
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// HiOpen ドライバオープン処理
        /// </summary>
        /// <param name="handle">オープンで取得するハンドル</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns></returns>
        public virtual UInt32 HiOpen(ref UInt32 dwAPIStatus, UInt32 dwTimeout)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                string target = ""; //targetを作成
                fnSetTarget(ref target);    //target tagから取得
                //Open処理コール
                dwAPIStatus = dyRsiClass.Open(target, dwTimeout);
                LOG(edge_FW.EG_LOG_DBG, "End HiOpen target[" + target + "] Status[" + dwAPIStatus.ToString("x8") + "H]");
                return fnRetValue(dwAPIStatus);
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// HiClose ドライバクローズ処理
        /// </summary>
        /// <param name="handle">オープンで取得するハンドル</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns></returns>
        public virtual UInt32 HiClose(ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ
                                                                //Close処理コール
                dwAPIStatus = dyRsiClass.Close();
                return fnRetValue(dwAPIStatus); //戻り値変換
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの読込 単体処理
        /// </summary>
        /// <param name="handle">ハンドル値</param>
        /// <param name="woID">ID</param>
        /// <param name="address">アドレス</param>
        /// <param name="readsize">読み込みサイズ</param>
        /// <param name="pData">読み込み時格納先バイト配列</param>
        /// <param name="buffersize">読込バッファサイズ</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns>ドライバ読込処理</returns>
        public virtual UInt32 HiRead(UInt16 woID, UInt32 address, UInt32 readsize, ref byte[] pData, UInt32 buffersize, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //Read処理コール
                if (address == 0)//ファイル

                dwAPIStatus = dyRsiClass.Read(address, ref pData, buffersize, dwTimeout);
                return fnRetValue(dwAPIStatus);//戻り値変換
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                pData = null;
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの読込 オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_IN_List"></param>
        /// <param name="tagRef_IN_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public virtual UInt32 HiReadEx(UInt16 woID, List<EgHgTagRef> tagRefs_IN, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //未実装
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : Not Implemented");
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの読込 オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_IN_List"></param>
        /// <param name="tagRef_IN_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public virtual UInt32 HiReadExAll(UInt16 woID, List<EgHgTagRef> tagRefs_IN, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //未実装
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : Not Implemented");
                return 0;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの書き込み 単体処理
        /// </summary>
        /// <param name="handle">ハンドル値</param>
        /// <param name="woID">ID</param>
        /// <param name="address">アドレス</param>
        /// <param name="writesize">書込みサイズ</param>
        /// <param name="pData">書き込みデータ</param>
        /// <param name="buffersize">バッファサイズ</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns></returns>
        public virtual UInt32 HiWrite(UInt16 woID, UInt32 address, UInt32 writesize, ref byte[] pData, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //Write処理コール
                dwAPIStatus = dyRsiClass.Write(address, ref pData, writesize, dwTimeout);
                return fnRetValue(dwAPIStatus);//戻り値変換

            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                pData = null;
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの書き込み オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public virtual UInt32 HiWriteEx(UInt16 woID, List<EgHgTagRef> tagRefs_OUT, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //未実装
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : Not Implemented");
                return 0;

            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// データの書き込み オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public virtual UInt32 HiWriteExAll(UInt16 woID, List<EgHgTagRef> tagRefs_OUT, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //未実装
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : Not Implemented");
                return 0;

            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }
        /// <summary>
        /// 再接続用関数
        /// </summary>
        /// <param name="handle">ハンドル値</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns></returns>
        public virtual UInt32 HiReConnect(ref UInt32 dwAPIStatus, UInt32 dwTimeout)
        {
            //CloseしてOpenする
            HiClose(ref dwAPIStatus);
            return HiOpen(ref dwAPIStatus, dwTimeout);
        }
        /// <summary>
        /// 接続チェック用関数
        /// </summary>
        /// <param name="handle">ハンドル値</param>
        /// <param name="dwAPIStatus">APIステータス</param>
        /// <returns></returns>
        public virtual bool HiConnectChk(ref UInt32 dwAPIStatus, UInt32 dwTimeout)
        {
            //デフォルト
            return true;
        }

        /// <summary>
        /// 使用するタグ名を生成する
        /// </summary>
        public virtual void fnServiceTagNameCreate()
        {
            //overwrite側で設定
        }

        /// <summary>
        /// 初期化設定文字列を作成
        /// </summary>
        /// <param name="apiConfig"></param>
        public virtual void fnSetConfig(ref string apiConfig)
        {
            try
            {
                //overwrite側で設定
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Target文字列を作成
        /// </summary>
        /// <param name="apiTarget"></param>
        public virtual void fnSetTarget(ref string apiTarget)
        {
            try
            {
                //overwrite側で設定
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// TagのAddressプロパティを取得
        /// </summary>
        /// <param name="TagName"></param>
        /// <param name="stValue"></param>
        /// <param name="defValue"></param>
        public void fnGetPropertyString(string TagName, ref string stValue, string defValue)
        {
            try
            {
                object value = new object();
                int result = EGAPI.EgTagGetProperty(TagName, "Address", ref value);//アドレス属性取得
                if (result == 0)//取得できた場合は戻り値に値をセット
                    stValue = (string)value;
                else            //取得できない場合はデフォルト値
                    stValue = defValue;                            // デフォルト
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// APIのステータス毎に戻り値を決定
        /// </summary>
        /// <param name="dwAPIStatus"></param>
        /// <returns></returns>
        public UInt32 fnRetValue(UInt32 dwAPIStatus)
        {
            if (dwAPIStatus == 0) //0は成功
                return CB_STS_OK;
            if (dwAPIStatus >= 0x80000000)
                return CB_STS_FATALERR; //復旧不可能/致命的なエラー

            return CB_STS_ERROR;//それ以外は通常エラー(復旧可能エラー) 処理は止めない
        }
        /// <summary>
        /// メソッドの存在チェック
        /// </summary>
        /// <param name="Func"></param>
        /// <param name="stFuncName"></param>
        /// <returns></returns>
        bool fnClsChkMethod(string stFuncName)
        {
            if (ClassType == null) return false; //関数呼び出し順に誤り

            MethodInfo Func = ClassType.GetMethod(stFuncName);   // DLL 関数取得
            if (Func == null)//ない場合はエラー
            {
                // 指定関数のロード失敗
                LOG(edge_FW.EG_LOG_ERR, "DLL " + stFuncName + " Function Load Fiald [" + HIAPI_RSI_DLLNAME + "]. FuncName [" + stFuncName + "]\n");  // ログ
                return false;
            }
            //存在する為True
            return true;
        }
        ///*****************************************************************************
        /// <summary>
        /// [DEBUG用] メッセージ出力
        /// </summary>
        /// <param name="message"></param>
        ///*****************************************************************************
        public void LOG(string Mode, string message)
        {
            try
            {
                Console.WriteLine(EGAPI.EGFW.ServiceRealName + " : " + Mode + " : " + message);
                EGAPI.EGFW.PrintLog(Mode, message);
            }
            catch
            { }
        }
        /// <summary>
        /// オブジェクト型の比較
        /// </summary>
        /// <param name="src"></param>
        /// <param name="dst"></param>
        /// <returns></returns>
        public static bool fnIsEqualObj(object src, object dst)
        {
            try
            {
                if (src == null && dst == null) //両方nullの場合はtrue
                    return true;
                if (src == null || dst == null) //どちらか一方がnullの場合はfalse;
                    return false;
                if (src.GetType() != dst.GetType())  //型が違う場合はfalse;
                    return false;

                if (true == src.Equals(dst)) //同じ場合はtrue 値型なのでこの関数でOK 値じゃない場合は要検討必要
                    return true;

                return false;//違う場合はfalse
            }
            catch
            {
                return false;
            }
        }
    }
}
