﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
//-----------------------------------------------------------------------
//
// 外部通信コンテナ FTP薄皮処理部
//
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using RTedge;   // RT-edge system API
using ExtCommon;
using System.Diagnostics;

namespace eWrap
{
    /// <summary>
    /// 薄皮処理 HiAPI呼び出しクラス
    /// </summary>
    public class HiApiCall : eWrapCls
    {
        //独自定義
        public const string ARG_ASCII_MODE = "ASCII";           //FTP転送時のアスキーモード 引数に渡す文字列
        public const string ARG_PASV_MODE = "PASV";             //FTP転送時のパッシブモード 引数に渡す文字列
        public const string ARG_SSL_MODE = "SSL";               //SSL有効(無効)モード 引数に渡す文字列
        public const string ARG_KEEPALIVE_MODE = "KeepAlive";   //要求の完了後に FTP サーバーへの制御接続を閉じるかどうか 引数に渡す文字列
        public const string ARG_LOCALFOLDER = "LocalFolder";    //FTP通信を行う作業用ローカルフォルダ 引数に渡す文字列
        public const string ARG_USERNAME = "User";              //ユーザー名 引数に渡す文字列
        public const string ARG_PASSWORD = "Password";          //パスワード 引数に渡す文字列

        public const UInt32 UPPathArea_StartAddr = 0x2000;      // アップロードファイルパス領域 先頭アドレス

        //本DLLで使用するタグ名定義
        public string PROP_MYSERV_ASCIIMODE;                    // FTP転送時のアスキーモード(またはバイナリモード)の指定
        public string PROP_MYSERV_PASVMODE;                     // FTP転送時のパッシブモード(またはアクティブモード)の指定
        public string PROP_MYSERV_SSLMODE;                      // SSL有効(無効)の指定
        public string PROP_MYSERV_KEEPALIVEMODE;                // 要求の完了後に FTP サーバーへの制御接続を閉じるかどうかを指定
        public string PROP_MYSERV_LOCALFOLDER;                  // FTP通信を行う作業用ローカルフォルダ
        public string PROP_MYSERV_URL;                          // FTPサーバー接続URL
        public string PROP_MYSERV_USERNAME;                     // ユーザー名
        public string PROP_MYSERV_PASSWORD;                     // パスワード

        public HiApiCall() : base()
        {
            base.HIAPI_RSI_DLLNAME = "HiFTP.dll";
            base.HIAPI_RSI_CLASSNAME = "HiFTP";


        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HiApiCall(edge_API _EGAPI) : base(_EGAPI)
        {
            base.HIAPI_RSI_DLLNAME = "HiFTP.dll";
            base.HIAPI_RSI_CLASSNAME = "HiFTP";
        }
        /// <summary>
        /// HiInit クラス初期化
        /// </summary>
        /// <param name="dwAPIStatus">APIのステータスを戻す</param>
        /// <returns>ステータス</returns>
        public override void HiClassInit(object _EGAPI, object _OutputDifferentFlag)
        {
            try
            {
                base.EGAPI = (edge_API)_EGAPI;
                base.OutputDifferentFlag = (byte)_OutputDifferentFlag;
            }
            catch (Exception ex)
            {
                Debug.WriteLine(System.Reflection.MethodBase.GetCurrentMethod().Name + " : 4");
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// データの読込 オブジェクト渡し 詳細処理実装 override
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_IN_List"></param>
        /// <param name="tagRef_IN_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override  UInt32 HiReadEx(UInt16 woID, List<EgHgTagRef> tagRefs_IN, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                bool bIsFile = true;
                byte[] Data = null;
                UInt32 offset = 0;
                UInt32 size = 0;
                //一つずつファイル作成 tagRefs_IN[IndexTagRef].

                //IOPortがvalidの場合は情報領域
                if (tagRef_IN_IOPorts[IndexIOPort].IsValied == 1)//情報領域
                    bIsFile = false;

                if (bIsFile)//ファイルの場合はAddressからファイルパスを取得
                {
                    if (tagRefs_IN[IndexTagRef].Address == null || tagRefs_IN[IndexTagRef].Address.Length == 0)//ファイルパス指定がない場合はエラー
                    {
                        dwAPIStatus = EW_NOADDRESS;
                        return CB_STS_FATALERR;//エラー
                    }
                    Data = System.Text.Encoding.ASCII.GetBytes(tagRefs_IN[IndexTagRef].Address);//バイト配列に置換
                    offset = 0;
                    size = (uint)Data.Length;
                }
                else//情報領域
                {
                    offset = (uint)tagRef_IN_IOPorts[IndexIOPort].Offset;
                    size = tagRefs_IN[IndexTagRef].Size;
                    Data = new byte[size];
                }
                //API実行
                dwAPIStatus = dyRsiClass.Read(offset, ref Data, size, dwTimeout);//APIをコール
                if(0 == dwAPIStatus && bIsFile == false)//正常終了時でかつ情報領域の場合はTagに書き込む
                {
                    object objData = egCommonFunc.GetTagAnyData(Data, (EGDEFINE.egTagDataType)tagRefs_IN[IndexTagRef].Type);
                    if (objData != null)
                        dwAPIStatus = (uint)EGAPI.EgTagWriteByEntry(tagRefs_IN[IndexTagRef].Entry, objData);
                    else
                        dwAPIStatus = EW_ILLEGALDATA;
                }
                return fnRetValue(dwAPIStatus);//戻り値変換
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// データの書き込み オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override UInt32 HiWriteEx(UInt16 woID, List<EgHgTagRef> tagRefs_OUT, int IndexTagRef, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, int IndexIOPort, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                //Write処理コール
                bool bIsFile = true;
                byte[] Data = null;
                UInt32 offset = 0;
                UInt32 size = 0;
                //一つずつファイル作成

                //IOPortがvalidの場合は情報領域
                if (tagRef_OUT_IOPorts[IndexIOPort].IsValied == 1)//情報領域
                    bIsFile = false;

                if (bIsFile)//ファイルの場合はAddressからファイルパスを取得
                {
                    if (tagRefs_OUT[IndexTagRef].Address == null || tagRefs_OUT[IndexTagRef].Address.Length == 0)//ファイルパス指定がない場合はエラー
                    {
                        dwAPIStatus = EW_NOADDRESS;
                        return CB_STS_FATALERR;//エラー
                    }
                    Data = System.Text.Encoding.ASCII.GetBytes(tagRefs_OUT[IndexTagRef].Address);//バイト配列に置換
                    offset = UPPathArea_StartAddr;  //必ずWrite領域のオフセットの始まりを指定する 1ファイル指定の為
                    size = (uint)Data.Length;
                }
                else//情報領域
                {
                   //書き込み禁止
                }
                dwAPIStatus = dyRsiClass.Write(offset, ref Data, size, dwTimeout);//APIをコール
                return fnRetValue(dwAPIStatus);//戻り値変換
                
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// 使用するタグ名を生成する
        /// </summary>
        public override void fnServiceTagNameCreate()
        {
            // サービスプロパティタグ名
            PROP_MYSERV_ASCIIMODE = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".ASCIIMode"; ;              // FTP転送時のアスキーモード(またはバイナリモード)の指定
            PROP_MYSERV_PASVMODE = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".PASVMode"; ;                // FTP転送時のパッシブモード(またはアクティブモード)の指定
            PROP_MYSERV_SSLMODE = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".SSLMode"; ;                  // SSL有効(無効)の指定
            PROP_MYSERV_KEEPALIVEMODE = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".KeepAliveMode"; ;      // 要求の完了後に FTP サーバーへの制御接続を閉じるかどうかを指定
            PROP_MYSERV_LOCALFOLDER = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".LocalFolder"; ;          // FTP通信を行う作業用ローカルフォルダ
            PROP_MYSERV_URL = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".Url"; ;                          // FTPサーバー接続URL
            PROP_MYSERV_USERNAME = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".Username"; ;                // ユーザー名
            PROP_MYSERV_PASSWORD = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".Password"; ;                // パスワード
        }
        /// <summary>
        /// 設定文字列の結合
        /// </summary>
        /// <param name="str"></param>
        /// <param name="addStr"></param>
        public void fnAddString(ref string str, string addStr)
        {
            if (str.Length > 0)
                str += HIAPI_CONFIG_DELIMITER + addStr;
            else
                str = addStr;
        }
        /// <summary>
        /// 初期化設定文字列を作成
        /// </summary>
        /// <param name="apiConfig"></param>
        public override void fnSetConfig(ref string apiConfig)
        {
            try
            {
                int result = 0;             //戻り値
                object value = new object();//値戻り値

                apiConfig = "";
                // FTP転送時のアスキーモード(またはバイナリモード)の指定
                result = EGAPI.EgTagRead(PROP_MYSERV_ASCIIMODE, ref value);
                if (result == 0 && (string.Compare(value.ToString(), "1", true) == 0 || string.Compare(value.ToString(), "true", true) == 0))
                    fnAddString(ref apiConfig, ARG_ASCII_MODE);// FTP転送時のアスキーモードの指定 指定なしはバイナリモード
                
                // FTP転送時のパッシブモード(またはアクティブモード)の指定
                result = EGAPI.EgTagRead(PROP_MYSERV_PASVMODE, ref value);
                if (result == 0 && (string.Compare(value.ToString(), "1", true) == 0 || string.Compare(value.ToString(), "true", true) == 0))
                    fnAddString(ref apiConfig, ARG_PASV_MODE);// FTP転送時のパッシブモードの指定 指定なしはアクティブモード
                
                // SSL有効(無効)の指定
                result = EGAPI.EgTagRead(PROP_MYSERV_SSLMODE, ref value);
                if (result == 0 && (string.Compare(value.ToString(), "1", true) == 0 || string.Compare(value.ToString(), "true", true) == 0))
                    fnAddString(ref apiConfig, ARG_SSL_MODE);// SSL有効の指定  指定なしは無効

                // 要求の完了後に FTP サーバーへの制御接続を閉じるかどうかを指定
                result = EGAPI.EgTagRead(PROP_MYSERV_KEEPALIVEMODE, ref value);
                if (result == 0 && (string.Compare(value.ToString(), "1", true) == 0 || string.Compare(value.ToString(), "true", true) == 0))
                    fnAddString(ref apiConfig, ARG_KEEPALIVE_MODE);// 要求の完了後に FTP サーバーへの制御接続を閉じるかどうかを指定

                string stLocalFolder = "";
                fnGetPropertyString(PROP_MYSERV_LOCALFOLDER, ref stLocalFolder, ""); // FTP通信を行う作業用ローカルフォルダ
                if (stLocalFolder.EndsWith("/") == false && stLocalFolder.EndsWith("\\") == false)
                    stLocalFolder += "/";//末尾にスラッシュを加える
                if(stLocalFolder.Length > 0)
                    fnAddString(ref apiConfig, ARG_LOCALFOLDER + "=" + stLocalFolder);// FTP通信を行う作業用ローカルフォルダ

                string stUsername = "";
                fnGetPropertyString(PROP_MYSERV_USERNAME, ref stUsername, "");    // ユーザー名
                if(stUsername.Length > 0)
                    fnAddString(ref apiConfig, ARG_USERNAME + "=" + stUsername);// ユーザー名

                string stPassword = "";
                fnGetPropertyString(PROP_MYSERV_PASSWORD, ref stPassword, "");    // パスワード
                if (stPassword.Length > 0)
                    fnAddString(ref apiConfig, ARG_PASSWORD + "=" + stPassword);// パスワード
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Target文字列を作成
        /// </summary>
        /// <param name="apiTarget"></param>
        public override void fnSetTarget(ref string apiTarget)
        {
            try
            {
                apiTarget = "";
                string stUrl = "";
                fnGetPropertyString(PROP_MYSERV_URL, ref stUrl, "");              // FTPサーバー接続URL
                if(stUrl.Length > 0)
                    apiTarget = stUrl;// FTPサーバー接続URL 引数に渡す文字列
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
