﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
//-----------------------------------------------------------------------
//
// 外部通信コンテナ MQTT薄皮処理部
//
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Text;
using RTedge;   // RT-edge system API
using ExtCommon;

namespace eWrap
{
   
    /// <summary>
    /// 薄皮処理 HiAPI呼び出しクラス
    /// </summary>
    public class HiApiCall : eWrapCls
    {
        //独自定義
        public string ARG_USEWEBSOCKET = "UseWebSocket";    // 通信方式の決定 bool
        public string ARG_USER = "User";                    // ユーザ名 Address
        public string ARG_PASS = "Pass";                    // パスワード Address
        public string ARG_SERVERSETTING = "ServerSetting";  // サーバーセッティング IP:Port Address
        
        //本DLLで使用するタグ名定義
        public string PROP_MYSERV_USEWEBSOCKET;             // 通信方式の決定 bool
        public string PROP_MYSERV_USER;                     // ユーザ名 Address
        public string PROP_MYSERV_PASS;                     // パスワード Address
        public string PROP_MYSERV_SERVERSETTING;            // サーバーセッティング IP:Port Address

        // タイムアウト
        public const UInt32 WAIT_FOREVER = 0xffffffff;
        public const UInt32 NO_WAIT = 0;

        // edge I/O 空間
        public const UInt32 EDGE_IO_INPUT = 0x0000;         //入力情報
        public const UInt32 EDGE_IO_OUTPUT = 0x2000;        //出力情報
        public const UInt32 EDGE_IO_INFORMATION = 0x4000;   //情報領域

        public Dictionary<string, string> dicTagRefOuts = new Dictionary<string, string>();
        public HiApiCall() : base()
        {
            base.HIAPI_RSI_DLLNAME = "HiMQTT.dll";
            base.HIAPI_RSI_CLASSNAME = "HiMQTT";
        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HiApiCall(edge_API _EGAPI) : base(_EGAPI)
        {
            base.HIAPI_RSI_DLLNAME = "HiMQTT.dll";
            base.HIAPI_RSI_CLASSNAME = "HiMQTT";
        }
        /// <summary>
        /// HiInit クラス初期化
        /// </summary>
        /// <param name="dwAPIStatus">APIのステータスを戻す</param>
        /// <returns>ステータス</returns>
        public override void HiClassInit(object _EGAPI, object _OutputDifferentFlag)
        {
            try
            {
                base.EGAPI = (edge_API)_EGAPI;
                base.OutputDifferentFlag = (byte)_OutputDifferentFlag;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        public const UInt32 ER_HI_TIMEOUT = 0xC0000003;
        /// <summary>
        /// データの読込 オブジェクト渡し 詳細処理実装 override
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_IN_List"></param>
        /// <param name="tagRef_IN_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override  UInt32 HiReadExAll(UInt16 woID, List<EgHgTagRef> tagRefs_IN, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ
                if (tagRefs_IN.Count < 1) return CB_STS_OK;     //タグが無い場合は処理終了

                // 入力情報をreadする
                var subDic = new Dictionary<string, string>();  //タグ名、値 のディクショナリを作成
                byte[] buffer = new byte[8192];
                dwAPIStatus = dyRsiClass.Read(EDGE_IO_INPUT, ref buffer, 0, NO_WAIT);//APIをコール
                if (dwAPIStatus == 0 || dwAPIStatus == ER_HI_TIMEOUT)   //MQTTではTimeoutは一般的なため無視する
                {
                    dwAPIStatus = 0;                                    //MQTTではTimeoutは一般的なため無視する

                    // データがある場合はdictionaryに格納
                    string subs = Encoding.GetEncoding("shift_jis").GetString(buffer).TrimEnd('\0');
                    // subs = "Hello=888|test=999"と返る
                    if (subs.Length > 0)
                    {
                        foreach (string sub in ItemValues.StringSplitOptions(subs, '|'))  // '|' で区切られた文字列を処理する
                        {
                            string[] subSplits = sub.Split('=');// = で分解
                            if(subSplits.Length > 1)//二つ以上要素がある場合のみ追加
                            {
                                subDic.Add(subSplits[0], subSplits[1]);
                            }
                        }
                    }
                }

                UInt32 result = 0;
                Int32 IndexTagRef = 0; //TagRefのインデックス
                bool bIsInfoArea = false; //情報領域か否か
                // ---------------------------------------------------------------------
                // 入力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < tagRef_IN_IOPorts.Count; i++)
                {
                    IndexTagRef = tagRef_IN_IOPorts[i].Index;//ソート前のindex

                    //IOPortがvalidの場合は情報領域
                    if (tagRef_IN_IOPorts[i].IsValied == 1)//情報領域かどうか
                        bIsInfoArea = true;

                    UInt32 offset = (uint)tagRef_IN_IOPorts[i].Offset;
                    UInt32 size = tagRefs_IN[IndexTagRef].Size;
                    byte[] Data = new byte[size];
                    if (bIsInfoArea == false) //情報領域では無い場合、Dictionalyから値があるか探す
                    {
                        // 何かしらデータがある場合は探す
                        if (subDic.Count > 0)
                        {
                            if(subDic.ContainsKey(tagRefs_IN[IndexTagRef].Name) == true) //データがある
                                dwAPIStatus = fnTagWrite(tagRefs_IN[IndexTagRef], subDic[tagRefs_IN[IndexTagRef].Name]);    //型に合わせて書き込み
                        }
                    }
                    else//情報領域
                    {
                        //API実行
                        ushort type = tagRefs_IN[IndexTagRef].Type;
                        dwAPIStatus = dyRsiClass.Read(offset, ref Data, size, dwTimeout);//APIをコール
                        if (0 == dwAPIStatus)//正常終了時でかつ情報領域の場合はTagに書き込む
                        {
                            //型がstringの場合の処理
                            if (type == (ushort)EGDEFINE.egTagDataType.String)
                            {
                                //byte配列から文字列を取得
                                object strObj = egCommonFunc.GetTagAnyData(Data, EGDEFINE.egTagDataType.String);
                                if(strObj != null)//変換後 null ではない場合は stringに変換して渡す
                                    dwAPIStatus = (uint)EGAPI.EgTagWriteString(tagRefs_IN[IndexTagRef].Name, (string)strObj);
                            }
                            else//型がstringではない場合
                                dwAPIStatus = (uint)EGAPI.EgTagWriteByEntry(tagRefs_IN[IndexTagRef].Entry, Data);
                        }
                    }
                    result |= fnRetValue(dwAPIStatus);//戻り値変換
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// データの書き込み オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override UInt32 HiWriteExAll(UInt16 woID, List<EgHgTagRef> tagRefs_OUT, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                //Write処理コール
                UInt32 result = 0;
                Int32 IndexTagRef = 0; //TagRefのインデックス
                bool bIsInfoArea = false; //情報領域か否か
                string pubs = "";         //書き込む値
                // ---------------------------------------------------------------------
                // 出力タグ更新処理 (TagRefs_OUT)
                // ---------------------------------------------------------------------
                for (int i = 0; i < tagRef_OUT_IOPorts.Count; i++)
                {
                    IndexTagRef = tagRef_OUT_IOPorts[i].Index;//ソート前のindex
                    //IOPortがvalidの場合は情報領域
                    if (tagRef_OUT_IOPorts[IndexTagRef].IsValied == 1)//情報領域
                        bIsInfoArea = true;

                    // オリジナルタグ名の取得
                    string tmpValue = "";
                    if(dicTagRefOuts.TryGetValue(tagRefs_OUT[IndexTagRef].Name, out tmpValue) == false)//存在しない場合は格納
                    {
                        object orgName = new object();
                        // エントリーを指定して名前を取得
                        dwAPIStatus = (uint)EGAPI.EgTagGetPropertyByEntry(tagRefs_OUT[IndexTagRef].Entry, "Name", ref orgName);
                        if(dwAPIStatus == 0)//成功時のみ格納
                            dicTagRefOuts.Add(tagRefs_OUT[IndexTagRef].Name, (string)orgName);
                        else//失敗時は元の名前を格納
                            dicTagRefOuts.Add(tagRefs_OUT[IndexTagRef].Name, tagRefs_OUT[IndexTagRef].Name);
                    }

                    if (bIsInfoArea == false)//データ領域の場合
                    {
                        object value = new object();//値戻り値
                        //  信方式の決定
                        dwAPIStatus = (uint)EGAPI.EgTagReadByEntry(tagRefs_OUT[IndexTagRef].Entry, ref value);
                        if (result == 0)
                        {
                            //値が同じ場合でも書き込むフラグの場合、または前の値と違っていたら書き込む
                            if (0 == base.OutputDifferentFlag)
                            {
                                if (eWrapCls.fnIsEqualObj(value, EGAPI.EGFW.tagRef_OUT_List[IndexTagRef].OldVal) == false)  //以前と値が違う場合は以下に入る
                                {
                                    fnAddString(ref pubs, dicTagRefOuts[tagRefs_OUT[IndexTagRef].Name] + "=" + value.ToString());// "|"区切りで文字列を追加
                                    EGAPI.EGFW.tagRef_OUT_List[IndexTagRef].OldVal = value;
                                }
                            }
                            else
                                fnAddString(ref pubs, dicTagRefOuts[tagRefs_OUT[IndexTagRef].Name] + "=" + value.ToString());// "|"区切りで文字列を追加
                        }
                        result |= fnRetValue(dwAPIStatus);//戻り値変換
                    }
                    else//情報領域
                    {
                        //書き込み禁止
                    }
                }
                if(pubs.Length > 0) //書き込む文字列がある場合はWrite呼び出し
                {
                    var buf = Encoding.GetEncoding("shift_jis").GetBytes(pubs);
                    dwAPIStatus = dyRsiClass.Write(EDGE_IO_OUTPUT, ref buf, (uint)buf.Length, dwTimeout);//APIをコール
                    result |= fnRetValue(dwAPIStatus);//戻り値変換
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// 使用するタグ名を生成する
        /// </summary>
        public override void fnServiceTagNameCreate()
        {
            // サービスプロパティタグ名
            PROP_MYSERV_USEWEBSOCKET = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".UseWebSocket";       // 通信方式の決定 bool
            PROP_MYSERV_USER = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".User";                       // ユーザ名 Address
            PROP_MYSERV_PASS = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".Pass";                       // パスワード Address
            PROP_MYSERV_SERVERSETTING = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".ServerSetting";     // サーバーセッティング IP:Port Address
            
        }
        /// <summary>
        /// 設定文字列の結合
        /// </summary>
        /// <param name="str"></param>
        /// <param name="addStr"></param>
        public void fnAddString(ref string str, string addStr)
        {
            if (str.Length > 0)
                str += HIAPI_CONFIG_DELIMITER + addStr;
            else
                str = addStr;
        }
        /// <summary>
        /// 初期化設定文字列を作成
        /// </summary>
        /// <param name="apiConfig"></param>
        public override void fnSetConfig(ref string apiConfig)
        {
            try
            {
                int result = 0;             //戻り値
                object value = new object();//値戻り値

                apiConfig = "";
                //  信方式の決定
                result = EGAPI.EgTagRead(PROP_MYSERV_USEWEBSOCKET, ref value);
                if (result == 0 && (bool)value == true)
                    fnAddString(ref apiConfig, ARG_USEWEBSOCKET);              // 通信方式の決定

                // ユーザー名
                string stUser = "";
                fnGetPropertyString(PROP_MYSERV_USER, ref stUser, "");    // ユーザー名
                if (stUser.Length > 0)
                    fnAddString(ref apiConfig, ARG_USER + "=" + stUser);// 文字列追加

                // パスワード
                string stPass = "";
                fnGetPropertyString(PROP_MYSERV_PASS, ref stPass, "");    // パスワード
                if (stUser.Length > 0)
                    fnAddString(ref apiConfig, ARG_PASS + "=" + stPass);// 文字列追加
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Target文字列を作成
        /// </summary>
        /// <param name="apiTarget"></param>
        public override void fnSetTarget(ref string apiTarget)
        {
            try
            {
                LOG(edge_FW.EG_LOG_DBG, "SetTarget TagRef_IN_List Count = " + base.EGAPI.EGFW.tagRef_IN_List.Count.ToString());
                
                apiTarget = "";
                string stServerSetting = "";                                              // サーバーセッティング IP:Port
                fnGetPropertyString(PROP_MYSERV_SERVERSETTING, ref stServerSetting, "");  // Addressから取得
                if (stServerSetting.Length > 0)
                    fnAddString(ref apiTarget, stServerSetting);                         //  IP:Port

                for (int i = 0; i < base.EGAPI.EGFW.tagRef_IN_List.Count; i++)          // 
                {
                    fnAddString(ref apiTarget, base.EGAPI.EGFW.tagRef_IN_List[i].Name);
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// edgeの型に合わせて書き込み
        /// </summary>
        /// <param name="tagRef"></param>
        /// <param name="strValue"></param>
        public uint fnTagWrite(EgHgTagRef edgetag, string strValue)
        {
            uint ret = 0;
            try
            {
                // 格納先Edgeのタグデータ型に合わせて分岐します
                switch (edgetag.Type)
                {
                    case (ushort)EGDEFINE.egTagDataType.Boolean:
                        try
                        {
                            // "true","false"の文字データをコンバートしてEdgeタグの値を更新する
                            object bval = Convert.ToBoolean(strValue);
                            ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, bval);
                        }
                        catch
                        {
                            // "0","!0"の値をコンバートしてEdgeタグの値を更新する
                            var bval = Convert.ToInt16(strValue);
                            ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, (bval != 0) ? true : false);
                        }
                        break;
                    case (ushort)EGDEFINE.egTagDataType.SByte:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object sbval = Convert.ToSByte(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, sbval);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.Byte:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object byval = Convert.ToByte(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, byval);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.Int16:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object int16array = Convert.ToInt16(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, int16array);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.UInt16:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object uint16array = Convert.ToUInt16(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, uint16array);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.Int32:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object int32array = Convert.ToInt32(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, int32array);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.UInt32:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object uint32array = Convert.ToUInt32(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, uint32array);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.Float:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する
                        object floatarray = Convert.ToSingle(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, floatarray);
                        break;
                    case (ushort)EGDEFINE.egTagDataType.Double:
                        // RTedgeデータ型にコンバートしてEdgeタグの値を更新する　double型は非サポートです
                        object doublearray = Convert.ToDouble(strValue);
                        ret = (uint)EGAPI.EgTagWriteByEntry(edgetag.Entry, doublearray);
                        break;
                }
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
            return ret;
        }
        /// <summary>
        /// 区切り文字separatorで区切られた文字列を分解してリストを作る
        /// </summary>
        public class ItemValues
        {
            public static string[] StringSplitOptions(string settings, char separator)
            {
                return settings.Split(separator);
            }
        }
    }
}
