﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
//-----------------------------------------------------------------------
//
// 外部通信コンテナ TCP薄皮処理部
//
//-----------------------------------------------------------------------
using System;
using System.Collections.Generic;
using RTedge;   // RT-edge system API
using ExtCommon;
namespace eWrap
{
   
    /// <summary>
    /// 薄皮処理 HiAPI呼び出しクラス
    /// </summary>
    public class HiApiCall : eWrapCls
    {
        //独自定義
        public string ARG_KEEPALIVE = "KeepAlive";                  // KeepAlive有効設定 bool
        public string ARG_KEEPALIVETIME = "KeepAliveTime";          // KeepAliveTime（単位:ms）uint32
        public string ARG_KEEPALIVEINTERVAL = "KeepAliveInterval";  // KeepAliveInterval（単位:ms）uint32
        public string ARG_LOCALIP = "LocalIP";    // 接続元IPアドレス、ポート Address
        public string ARG_REUSEADDR = "ReUseAddr";    // ポート再利用有効 bool
        public string ARG_TARGETIP = "TargetIP";    // 接続先IPアドレス、ポート ドメイン名指定可能 Address

        //本DLLで使用するタグ名定義
        public string PROP_MYSERV_KEEPALIVE;                        // KeepAlive有効設定 bool
        public string PROP_MYSERV_KEEPALIVETIME;                    // KeepAliveTime（単位:ms）uint32
        public string PROP_MYSERV_KEEPALIVEINTERVAL;                // KeepAliveInterval（単位:ms）uint32
        public string PROP_MYSERV_LOCALIP;                          // 接続元IPアドレス、ポート Address
        public string PROP_MYSERV_REUSEADDR;                        // ポート再利用有効 bool
        public string PROP_MYSERV_TARGETIP;                         // 接続先IPアドレス、ポート ドメイン名指定可能 Address

        public uint TCP_RECV_OFFSET = 0x0000;                       // 受信データ オフセット 設定
        public uint TCP_SEND_OFFSET = 0x2000;                       // 送信データ オフセット 設定
        public HiApiCall() : base()
        {
            base.HIAPI_RSI_DLLNAME = "HiTCP.dll";
            base.HIAPI_RSI_CLASSNAME = "HiTCPC";
        }
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public HiApiCall(edge_API _EGAPI) : base(_EGAPI)
        {
            base.HIAPI_RSI_DLLNAME = "HiTCP.dll";
            base.HIAPI_RSI_CLASSNAME = "HiTCPC";
        }
        /// <summary>
        /// HiInit クラス初期化
        /// </summary>
        /// <param name="dwAPIStatus">APIのステータスを戻す</param>
        /// <returns>ステータス</returns>
        public override void HiClassInit(object _EGAPI, object _OutputDifferentFlag)
        {
            try
            {
                base.EGAPI = (edge_API)_EGAPI;
                base.OutputDifferentFlag = (byte)_OutputDifferentFlag;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// データの読込 オブジェクト渡し 詳細処理実装 override
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_IN_List"></param>
        /// <param name="tagRef_IN_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override  UInt32 HiReadExAll(UInt16 woID, List<EgHgTagRef> tagRefs_IN, List<EgHgEdgeIOPort> tagRef_IN_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                UInt32 result = 0;
                bool bIsDataset = true;
                byte[] Data = null;
                UInt32 offset = 0;
                UInt32 size = 0;
                Int32 IndexTagRef = 0; //TagRefのインデックス
                bool bIsExecDsetFlag = false; //データセットでの処理が一度でも行われた場合は2回目以降は無視
                // ---------------------------------------------------------------------
                // 入力タグ更新 (TagRefs_IN) ※RTCDへの入力を自動処理します
                // ---------------------------------------------------------------------
                for (int i = 0; i < tagRef_IN_IOPorts.Count; i++)
                {
                    IndexTagRef = tagRef_IN_IOPorts[i].Index;//ソート前のindex

                    //IOPortがvalidの場合は情報領域
                    if (tagRef_IN_IOPorts[i].IsValied == 1)//情報領域
                        bIsDataset = false;


                    if (bIsDataset && bIsExecDsetFlag == false)//データセットの場合はAddressからデータセットを取得,2回目以降は処理しない
                    {
                        if (tagRefs_IN[IndexTagRef].Address == null || tagRefs_IN[IndexTagRef].Address.Length == 0)//データセット名がない場合はエラー
                        {
                            dwAPIStatus = EW_NOADDRESS;
                            return CB_STS_FATALERR;//エラー
                        }
                        //データセットがあるかチェック
                        string DatasetName = tagRefs_IN[IndexTagRef].Address;
                        // 読み込み用のバッファの生成
                        int dataSetSize = EGAPI.EgDatasetGetSize(DatasetName);
                        if (dataSetSize == 0)
                        {
                            dwAPIStatus = EW_DATASET_NOTFOUND;
                            return CB_STS_FATALERR;//エラー
                        }
                        byte[] buffer = new byte[dataSetSize + 2];  // バッファ先頭2byteにデータサイズが入るため+2
                        dwAPIStatus = dyRsiClass.Read(TCP_RECV_OFFSET, ref buffer, (uint)dataSetSize, dwTimeout);//APIをコール
                        result |= fnRetValue(dwAPIStatus);//戻り値変換
                        //正常に終了した場合はデータセットに書き込み実行
                        if (dwAPIStatus == 0)
                        {
                            // 先頭2byteに入っているデータサイズを除く
                            byte[] setBuffer = new byte[dataSetSize];
                            Array.Copy(buffer, 2, setBuffer, 0, dataSetSize);
                            // データセットに定義されているタグへ書込み
                            dwAPIStatus = (uint)EGAPI.EgDatasetSetBinary(DatasetName, setBuffer, (uint)dataSetSize);
                            result |= fnRetValue(dwAPIStatus);//戻り値変換
                            //エラーでも1回実行とみなす
                            bIsExecDsetFlag = true;
                        }                                                              //正常に終了した場合は送信実行
                    }
                    else//情報領域
                    {
                        offset = (uint)tagRef_IN_IOPorts[i].Offset;
                        size = tagRefs_IN[IndexTagRef].Size;
                        Data = new byte[size];
                        //API実行
                        dwAPIStatus = dyRsiClass.Read(offset, ref Data, size, dwTimeout);//APIをコール
                        if (0 == dwAPIStatus && bIsDataset == false)//正常終了時でかつ情報領域の場合はTagに書き込む
                        {
                            dwAPIStatus = (uint)EGAPI.EgTagWriteByEntry(tagRefs_IN[IndexTagRef].Entry, Data);
                        }
                        result |= fnRetValue(dwAPIStatus);//戻り値変換
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// データの書き込み オブジェクト渡し 詳細処理実装
        /// </summary>
        /// <param name="woID"></param>
        /// <param name="tagRef_OUT_List"></param>
        /// <param name="tagRef_OUT_IOPortList"></param>
        /// <param name="dwTimeout"></param>
        /// <returns></returns>
        public override UInt32 HiWriteExAll(UInt16 woID, List<EgHgTagRef> tagRefs_OUT, List<EgHgEdgeIOPort> tagRef_OUT_IOPorts, UInt32 dwTimeout, ref UInt32 dwAPIStatus)
        {
            try
            {
                //nullチェック
                if (dyRsiClass == null) return CB_STS_FATALERR;//既にコンストラクタでエラーを出しているためここでは処理しないだけ

                //Write処理コール
                UInt32 result = 0;
                bool bIsDataset = true;
                Int32 IndexTagRef = 0;  //TagRefのインデックス
                bool bIsExecDsetFlag = false; //データセットでの処理が一度でも行われた場合は2回目以降は無視

                // ---------------------------------------------------------------------
                // 出力タグ更新処理 (TagRefs_OUT)
                // ---------------------------------------------------------------------
                for (int i = 0; i < tagRef_OUT_IOPorts.Count; i++)
                {
                    IndexTagRef = tagRef_OUT_IOPorts[i].Index;//ソート前のindex
                    //IOPortがvalidの場合は情報領域
                    if (tagRef_OUT_IOPorts[IndexTagRef].IsValied == 1)//情報領域
                        bIsDataset = false;

                    if (bIsDataset && bIsExecDsetFlag == false)//データセットの場合はAddressからデータセットを取得,2回目以降は処理しない
                    {
                        if (tagRefs_OUT[IndexTagRef].Address == null || tagRefs_OUT[IndexTagRef].Address.Length == 0)//データセット名がない場合はエラー
                        {
                            dwAPIStatus = EW_NOADDRESS;
                            return CB_STS_FATALERR;//エラー
                        }
                        //データセットがあるかチェック
                        string DatasetName = tagRefs_OUT[IndexTagRef].Address;
                        // 書き込み用のバッファの生成
                        int dataSetSize = EGAPI.EgDatasetGetSize(DatasetName);
                        if (dataSetSize == 0)
                        {
                            dwAPIStatus = EW_DATASET_NOTFOUND;
                            return CB_STS_FATALERR;//エラー
                        }
                        byte[] buffer = new byte[dataSetSize];
                        dwAPIStatus = (uint)EGAPI.EgDatasetGetBinary(DatasetName, buffer, (uint)dataSetSize);
                        result |= fnRetValue(dwAPIStatus);//戻り値変換
                        //正常に終了した場合は送信実行
                        if (dwAPIStatus == 0)
                        {
                            dwAPIStatus = dyRsiClass.Write(TCP_SEND_OFFSET, ref buffer, (uint)dataSetSize, dwTimeout);//APIをコール
                            result |= fnRetValue(dwAPIStatus);//戻り値変換
                            //エラーでも1回実行とみなす
                            bIsExecDsetFlag = true;
                        }
                    }
                    else//情報領域
                    {
                        //書き込み禁止
                    }
                }
                return result;
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
                return CB_STS_FATALERR;
            }
        }

        /// <summary>
        /// 使用するタグ名を生成する
        /// </summary>
        public override void fnServiceTagNameCreate()
        {
            // サービスプロパティタグ名
            PROP_MYSERV_KEEPALIVE = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".KeepAlive";                         // KeepAlive有効設定
            PROP_MYSERV_KEEPALIVETIME = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".KeepAliveTime";                 // KeepAliveTime（単位:ms）
            PROP_MYSERV_KEEPALIVEINTERVAL = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".KeepAliveInterval";         // KeepAliveInterval（単位:ms）
            PROP_MYSERV_LOCALIP = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".LocalIP";                             // 接続元IPアドレス、ポート
            PROP_MYSERV_REUSEADDR = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".ReUseAddr";                         // ポート再利用有効
            PROP_MYSERV_TARGETIP = "SERVICE." + EGAPI.EGFW.ServiceRealName + ".TargetIP";                           // 接続先IPアドレス、ポート ドメイン名指定可能
        }
        /// <summary>
        /// 設定文字列の結合
        /// </summary>
        /// <param name="str"></param>
        /// <param name="addStr"></param>
        public void fnAddString(ref string str, string addStr)
        {
            if (str.Length > 0)
                str += HIAPI_CONFIG_DELIMITER + addStr;
            else
                str = addStr;
        }
        /// <summary>
        /// 初期化設定文字列を作成
        /// </summary>
        /// <param name="apiConfig"></param>
        public override void fnSetConfig(ref string apiConfig)
        {
            try
            {
                int result = 0;             //戻り値
                object value = new object();//値戻り値

                apiConfig = "";
                //  KeepAlive有効設定
                result = EGAPI.EgTagRead(PROP_MYSERV_KEEPALIVE, ref value);
                if (result == 0 && (bool)value == true)
                    fnAddString(ref apiConfig, ARG_KEEPALIVE);              // KeepAlive有効設定

                // KeepAliveTime（単位:ms）
                result = EGAPI.EgTagRead(PROP_MYSERV_KEEPALIVETIME, ref value);
                if (result == 0)
                    fnAddString(ref apiConfig, ARG_KEEPALIVETIME + "=" + value.ToString().ToLower());// KeepAliveTime（単位:ms）

                // KeepAliveInterval（単位:ms）
                result = EGAPI.EgTagRead(PROP_MYSERV_KEEPALIVEINTERVAL, ref value);
                if (result == 0)
                    fnAddString(ref apiConfig, ARG_KEEPALIVEINTERVAL + "=" + value.ToString().ToLower());// KeepAliveInterval（単位:ms）

                // 接続元IPアドレス、ポート
                string stLocalIP = "";
                fnGetPropertyString(PROP_MYSERV_LOCALIP, ref stLocalIP, "");    // 接続元IPアドレス、ポート
                if (stLocalIP.Length > 0)
                    fnAddString(ref apiConfig, ARG_LOCALIP + "=" + stLocalIP);// 文字列追加

                // ポート再利用有効
                result = EGAPI.EgTagRead(PROP_MYSERV_REUSEADDR, ref value);
                if (result == 0 && (bool)value == true)
                    fnAddString(ref apiConfig, ARG_REUSEADDR);              // ポート再利用有効
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
        /// <summary>
        /// Target文字列を作成
        /// </summary>
        /// <param name="apiTarget"></param>
        public override void fnSetTarget(ref string apiTarget)
        {
            try
            {
                apiTarget = "";
                string stIP = "";
                fnGetPropertyString(PROP_MYSERV_TARGETIP, ref stIP, "");              // 接続先IPアドレス、ポート ドメイン名
                if (stIP.Length > 0)
                    apiTarget = stIP;// 接続先IPアドレス、ポート ドメイン名
            }
            catch (Exception ex)
            {
                LOG(edge_FW.EG_LOG_ERR, System.Reflection.MethodBase.GetCurrentMethod().Name + " : " + ex.Message);
            }
        }
    }
}
