﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using Rsi;
using System;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace HiMqttTest
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        HiMQTT mqtt;      // Rsi-MQTT ライブラリ
        static bool RxRunning;

        public MainWindow()
        {
            InitializeComponent();
            mqtt = new HiMQTT();
        }

		// Init
        private void Button1_Click(object sender, RoutedEventArgs e)
        {
            var ret = mqtt.Init("MQTT", "User=machii|Pass=micronet|ClientId=test", 0);
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Open
        private void Button2_Click(object sender, RoutedEventArgs e)
        {
            var ret = mqtt.Open("127.0.0.1:1883|Hello|test", 2000);
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Close
        private void Button3_Click(object sender, RoutedEventArgs e)
        {
            var ret = mqtt.Close();
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Read
        private void Button4_Click(object sender, RoutedEventArgs e)
        {
            if(RxRunning)
                return;
            var task = Task.Run(() =>
            {
                RxRunning = true;
                while (true)
                {
                    var buf = new byte[100];
                    var ret = mqtt.Read(0x0000, ref buf, 0, HiMQTT.WAIT_FOREVER);
                    if (ret == 0)
                    {
                        var Rx = Encoding.GetEncoding("shift_jis").GetString(buf).TrimEnd('\0');
                        // Rx = "Hello=888|test=999"と返る

                        var message = string.Format("Length {0}\r\n{1}", Rx.Length, Rx);
                        MessageBox.Show(message);
                    }
                    else
                    {
                        var message = string.Format("Status {0:X}", ret);
                        MessageBox.Show(message);
                        break;
                    }
                }
                RxRunning = false;
            });
        }

        // Write
        private void Button5_Click(object sender, RoutedEventArgs e)
        {
            var Tx = "Hello=888|test=999|everyone=777";
            var buf = Encoding.GetEncoding("shift_jis").GetBytes(Tx);
            uint timeout = 1000;
            var ret = mqtt.Write(0x2000, ref buf, (uint)buf.Length, timeout);
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }
    }
}
