﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;

namespace Rsi
{
    public class Rsi
    {
        //--------------------------------------------------
        // 共通メソッドの定義
        //--------------------------------------------------
        public Rsi() { }
        public UInt32 Init(string ifname, string config, UInt32 dwTimeout) { return 0; }
        public UInt32 Open(string target, UInt32 dwTimeout) { return 0; }
        public UInt32 Read(UInt32 dwAddr, ref byte[] buffer, UInt32 dwSize, UInt32 dwTimeout) { return 0; }
        public UInt32 Write(UInt32 dwAddr, ref byte[] buffer, UInt32 dwSize, UInt32 dwTimeout) { return 0; }
        public UInt32 Close() { return 0; }

        //--------------------------------------------------
        // クラスメンバ変数の定義
        //--------------------------------------------------
        protected bool boInitialized = false;                     // 初期化状態フラグ (true:初期化完了, false:初期化未完了)
        protected bool boOpened = false;                          // オープン状態フラグ (true:オープン完了,false:オープン未完了)

        //--------------------------------------------------
        // ステータスコード宣言
        //--------------------------------------------------
        /* 区分:正常 */
        public const UInt32 ER_HI_OK = 0x00000000;

        /* 区分:再接続(再Open)を要するエラー */
        public const UInt32 ER_HI_NOT_INITED = 0xC0000000;
        public const UInt32 ER_HI_OPENED = 0xC0000001;
        public const UInt32 ER_HI_CONNECT = 0xC0000002;
        public const UInt32 ER_HI_TIMEOUT = 0xC0000003;
        public const UInt32 ER_HI_NOT_OPENED = 0xC0000004;
        public const UInt32 ER_HI_ADDR_RANGE = 0xC0000005;

        //--------------------------------------------------
        // 定数宣言
        //--------------------------------------------------
        /* アドレス位置定数 */
        public const UInt32 IArea_StartAddr = 0x00000000;           // 入力情報 先頭アドレス
        public const UInt32 IArea_Size = 0x00002000;                // 入力情報 バイトサイズ
        public const UInt32 QArea_StartAddr = 0x00002000;           // 出力情報 先頭アドレス
        public const UInt32 QArea_Size = 0x00002000;                // 出力情報 バイトサイズ
        public const UInt32 InfoArea_StartAddr = 0x00004000;        // 情報領域 先頭アドレス
        public const UInt32 InfoArea_Size = 0x00001028;             // 情報領域 終了アドレス

        /* 接続ステータス */
        public const UInt32 STATUS_DISCONNECT = 0;
        public const UInt32 STATUS_CONNECTING = 1;
        public const UInt32 STATUS_ESTABLISH = 2;

        /* タイムアウト */
        public const UInt32 WAIT_FOREVER = 0xffffffff;
        public const UInt32 NO_WAIT = 0;
    }

    public partial class HiTCPC : Rsi
    {
        /// <summary>
        /// インターフェースの初期化
        /// </summary>
        /// <param name="ifname">インターフェース名文字列</param>
        /// <param name="config">コンフィグ文字列</param>
        /// <param name="dwTimeout">初期化完了までの制限時間msec</param>
        /// <returns>エラーコード</returns>
        public new UInt32 Init(string ifname, string config, UInt32 dwTimeout)
        {
            //--------------------------------------------------
            // APIコール回数 統計情報更新
            //--------------------------------------------------
            ++Inf_Table.init_cnt;

            //--------------------------------------------------
            // Init済はスキップ成功
            //--------------------------------------------------
            if (boInitialized == true)
                return ER_HI_OK;     // 既に処理済は正常終了

            //--------------------------------------------------
            // コンフィグパラメータ解釈処理
            //--------------------------------------------------
            var ret1 = fnTreatConfig(config);
            if (ret1 != ER_HI_OK)
                return ret1;

            //--------------------------------------------------
            // インターフェース初期化要求
            //--------------------------------------------------
            fnInitRequest();

            //--------------------------------------------------
            // インターフェース初期化完了の待機
            //--------------------------------------------------
            fnInitWait(dwTimeout);

            //--------------------------------------------------
            // 完了
            //--------------------------------------------------
            boInitialized = true;   // 初期化完了
            return ER_HI_OK;     // 正常終了
        }

        /// <summary>
        /// 通信のオープン
        /// </summary>
        /// <param name="target">ターゲット指定文字列</param>
        /// <param name="dwTimeout">オープン完了までの制限時間msec</param>
        /// <returns>エラーコード</returns>
        public new UInt32 Open(string target, UInt32 dwTimeout)
        {
            //--------------------------------------------------
            // APIコール回数 統計情報更新
            //--------------------------------------------------
            Inf_Table.open_cnt++;
            Inf_Table.status = STATUS_CONNECTING;       // 接続試行中

            //--------------------------------------------------
            // Initされていない場合
            //--------------------------------------------------
            if (boInitialized != true)
                return ER_HI_NOT_INITED;

            if (boOpened == false)
            {
                //--------------------------------------------------
                // ターゲットパラメータ解釈処理
                //--------------------------------------------------
                var ret0 = fnTreatTarget(target);
	            if (ret0 != ER_HI_OK)
	                return ret0;

                //--------------------------------------------------
                // 接続の要求
                //--------------------------------------------------
                var ret1 = fnOpenRequest();
                if (ret1 == false)
                    return ER_HI_CONNECT;

                //--------------------------------------------------
                // 接続の完了待ち
                //--------------------------------------------------
                var ret2 = fnOpenWait(dwTimeout);
                if (ret2 == -1)
                    return ER_HI_TIMEOUT;

                //--------------------------------------------------
                // 完了
                //--------------------------------------------------
                Inf_Table.status = STATUS_ESTABLISH;        // 接続状態
                boOpened = true;        // オープン中状態へ遷移
            }
            return ER_HI_OK;     // 正常終了
        }

        /// <summary>
        /// 通信のクローズ
        /// </summary>
        /// <returns>エラーコード</returns>
        public new UInt32 Close()
        {
            //--------------------------------------------------
            // APIコール回数 統計情報更新
            //--------------------------------------------------
            Inf_Table.close_cnt++;

            //--------------------------------------------------
            // Initされていない場合
            //--------------------------------------------------
            if (boInitialized != true)
                return ER_HI_NOT_INITED;

            if (boOpened == true)
            {
                //--------------------------------------------------
                // 切断の要求
                //--------------------------------------------------
                var ret1 = fnCloseRequest();

                //--------------------------------------------------
                // 切断の待機
                //--------------------------------------------------
                var ret2 = fnCloseWait(0);
                
	            //--------------------------------------------------
	            // 完了
	            //--------------------------------------------------
	            Inf_Table.status = STATUS_DISCONNECT;       // 未接続
	            boOpened = false;   // Open中状態を解除
            }

            return ER_HI_OK; // 正常終了
        }

        /// <summary>
        /// データリード
        /// </summary>
        /// <param name="dwAddr">レジスタ指定アドレス</param>
        /// <param name="buffer">データ格納先バッファ</param>
        /// <param name="dwSize">データサイズ指定byte</param>
        /// <param name="dwTimeout">データリード制限時間msec</param>
        /// <returns></returns>
        public new UInt32 Read(UInt32 dwAddr, ref byte[] buffer, UInt32 dwSize, UInt32 dwTimeout)
        {
            //--------------------------------------------------
            // APIコール回数 統計情報更新
            //--------------------------------------------------
            Inf_Table.read_cnt++;

            //--------------------------------------------------
            // Initされていない場合
            //--------------------------------------------------
            if (boInitialized != true)
                return ER_HI_NOT_INITED;

            //--------------------------------------------------
            // Open状態チェック
            //--------------------------------------------------
            if (boOpened != true)
                return ER_HI_NOT_OPENED;

            //--------------------------------------------------
            // アクセス対象エリア区分の判定
            //--------------------------------------------------
            var IArea = ((dwAddr >= IArea_StartAddr) && ((dwAddr) < IArea_StartAddr + IArea_Size));
            var QArea = ((dwAddr >= QArea_StartAddr) && ((dwAddr) < QArea_StartAddr + QArea_Size));
            var InfoArea = ((dwAddr >= InfoArea_StartAddr) && ((dwAddr) < InfoArea_StartAddr + InfoArea_Size));

            //--------------------------------------------------
            // データ受信
            //--------------------------------------------------
            if (IArea == true)
            {
                //--------------------------------------------------
                // 受信データ発生待ち
                //--------------------------------------------------
                var ret2 = fnReadWait(dwTimeout,(int)dwSize);
                // タイムアウトの発生
                if (ret2 == -1)
                    return ER_HI_TIMEOUT;
                // 切断の発生
                if (ret2 == -2)
                    return ER_HI_CONNECT;

                //--------------------------------------------------
                // 受信データの取り出し
                //--------------------------------------------------
                var ret1 = fnReadRequest( ref buffer, (int)dwSize);
                // 切断の発生
                if (ret1 == -2)
                    return ER_HI_CONNECT;

                //--------------------------------------------------
                // 完了
                //--------------------------------------------------
                return ER_HI_OK;
            }
            //--------------------------------------------------
            // 管理情報の取り出し
            //--------------------------------------------------
            else if ( InfoArea == true )
            {
                //--------------------------------------------------
                // 管理データの取り出し
                //--------------------------------------------------
                int offs = (int)(dwAddr - InfoArea_StartAddr);
                int size = (int)dwSize;
                var ret1 = fnReadInfo(ref buffer, offs ,size);

                //--------------------------------------------------
                // 完了
                //--------------------------------------------------
                return ER_HI_OK;
            }

            return ER_HI_ADDR_RANGE;
        }

        /// <summary>
        /// データライト
        /// </summary>
        /// <param name="dwAddr">レジスタ指定アドレス</param>
        /// <param name="buffer">データバッファ</param>
        /// <param name="dwSize">データサイズbyte</param>
        /// <param name="dwTimeout">データライト制限時間msec</param>
        /// <returns></returns>
        public new UInt32 Write(UInt32 dwAddr, ref byte[] buffer, UInt32 dwSize, UInt32 dwTimeout)
        {
            //--------------------------------------------------
            // APIコール回数 統計情報更新
            //--------------------------------------------------
            Inf_Table.write_cnt++;

            //--------------------------------------------------
            // Initされていない場合
            //--------------------------------------------------
            if (boInitialized != true)
                return ER_HI_NOT_INITED;

            //--------------------------------------------------
            // Open状態チェック
            //--------------------------------------------------
            if (boOpened != true)
                return ER_HI_NOT_OPENED;

            //--------------------------------------------------
            // アクセス対象エリア区分の判定
            //--------------------------------------------------
            var QArea = ((dwAddr >= QArea_StartAddr) && ((dwAddr) < QArea_StartAddr + QArea_Size));
            if (QArea == true)
            {
                //--------------------------------------------------
                // 出力の要求
                //--------------------------------------------------
                var ret1 = fnWriteRequest( buffer, (int)dwSize);
                // 切断の発生
                if (ret1 == -2)
                    return ER_HI_CONNECT;

                //--------------------------------------------------
                // 出力の完了待ち
                //--------------------------------------------------
                var ret2 = fnWriteWait(dwTimeout);
                // 切断の発生
                if (ret2 == -2)
                    return ER_HI_CONNECT;

                //--------------------------------------------------
                // 完了
                //--------------------------------------------------
                return ER_HI_OK;
            }

            return ER_HI_ADDR_RANGE;
        }
    }

    /* 情報領域用構造体 */
    [StructLayout(LayoutKind.Sequential, Pack = 1)]
    struct InfomationArea
    {
        public UInt32 status;

        public UInt32 init_cnt;
        public UInt32 open_cnt;
        public UInt32 close_cnt;
        public UInt32 read_cnt;
        public UInt32 write_cnt;

        public InfomationArea(string ini_uri)
        {
            status = 0;
            init_cnt = 0;
            open_cnt = 0;
            close_cnt = 0;
            read_cnt = 0;
            write_cnt = 0;
        }
    }
}
