﻿////
/// Copyright(C) 2020 Micronet Co. All Rights Reserved.
//
using Rsi;
using System;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;

namespace HiTcpTest
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        HiTCPC mTcp;      // Rsi-TCP ライブラリ
        static bool RxRunning;

        public MainWindow()
        {
            InitializeComponent();
            mTcp = new HiTCPC();
        }

		// Init
        private void Button1_Click(object sender, RoutedEventArgs e)
        {
            var ret = mTcp.Init("", "KeepAliveEnable|KeepAliveTime=3600000|KeepAliveInterval=5000|LocalIP=127.0.0.1", 0);
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Open
        private void Button2_Click(object sender, RoutedEventArgs e)
        {
            var ret = mTcp.Open("127.0.0.1:50000", 5000);
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Close
        private void Button3_Click(object sender, RoutedEventArgs e)
        {
            var ret = mTcp.Close();
            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }

        // Read
        private void Button4_Click(object sender, RoutedEventArgs e)
        {
            if (RxRunning)
                return;
            var task = Task.Run(() =>
            {
                RxRunning = true;
                while (true)
                {
                    var buf = new byte[2000];
                    uint addr = 0x0000;
                    uint size = 0;
                    //uint addr = 0x4008;
                    //uint size = 8;
                    uint timeout = 1000;
                    var ret = mTcp.Read(addr, ref buf, size, timeout);
                    if (ret == 0)
                    {
                        if (addr >= 0x4000)
                        {
                            var Val = BitConverter.ToInt64(buf.Take((int)size).ToArray(),0);
                            var message = string.Format("{0}", Val);
                            MessageBox.Show(message);
                        }
                        else
                        {
                            // 受信したサイズ(UInt16)の取り出し。先頭2byte
                            // 受信したデータ(byte[])の取り出し。先頭2byte以降～RxSizeバイト
                            UInt16 RxSize = BitConverter.ToUInt16(buf, 0);
                            byte[] RxData = buf.Skip(2).Take(RxSize).ToArray();
                            var message = string.Format("Length {0}\r\n{1}", RxSize, BitConverter.ToString(RxData));
                            MessageBox.Show(message);
                        }
                        break;
                    }
                    else
                    {
                        var message = string.Format("Status {0:X}",ret);
                        MessageBox.Show(message);
                        break;
                    }
                }
                RxRunning = false;
            });
        }

        // Write
        private void Button5_Click(object sender, RoutedEventArgs e)
        {
            string text = "Hello World!";
            byte[] byBuff = null;

            //ASCII エンコード
            byBuff = System.Text.Encoding.ASCII.GetBytes(text);

            var ret = mTcp.Write(0x2000,ref byBuff, (uint)byBuff.Length, 3000);

            var message = string.Format("Status {0:X}", ret);
            MessageBox.Show(message);
        }
    }
}
